@@ -17,11 +17,19 @@
 # limitations under the License.
 #
 set -e
-export CC="gcc-4.8"
-export CXX="g++-4.8"
-wget https://s3.amazonaws.com/download.draios.com/dependencies/cmake-3.3.2.tar.gz
-tar -xzf cmake-3.3.2.tar.gz
-cd cmake-3.3.2
+
+arch="$(uname -i)"
+
+if [[ "$arch" == "s390x" ]] || [[ "$arch" == "ppc64le" ]]; then
+	export CC="gcc-7"
+	export CXX="g++-7"
+else
+	export CC="gcc-4.8"
+	export CXX="g++-4.8"
+fi
+wget https://github.com/Kitware/CMake/releases/download/v3.16.4/cmake-3.16.4.tar.gz
+tar -xzf cmake-3.16.4.tar.gz
+cd cmake-3.16.4
 ./bootstrap --prefix=/usr
 make
 sudo make install
@@ -31,5 +39,6 @@ cd build
 cmake .. -DCMAKE_BUILD_TYPE=$BUILD_TYPE
 make VERBOSE=1
 make package
+make run-unit-tests
 cd ..
 test/sysdig_trace_regression.sh build/userspace/sysdig/sysdig build/userspace/sysdig/chisels $TRAVIS_BRANCH
