@@ -741,7 +741,7 @@ static __always_inline int __bpf_val_to_ring(struct filler_data *data,
 			res = bpf_probe_read_str(&data->buf[data->state->tail_ctx.curoff & SCRATCH_SIZE_HALF],
 						 PPM_MAX_ARG_SIZE,
 						 (const void *)val);
-			if (res < 0)
+			if (res == -EFAULT)
 				return PPM_FAILURE_INVALID_USER_MEMORY;
 			len = res;
 		} else {
@@ -931,6 +931,9 @@ static __always_inline int bpf_val_to_ring_type(struct filler_data *data,
 
 static __always_inline bool bpf_in_ia32_syscall()
 {
+#ifdef __ppc64__
+	return 0;
+#else
 	struct task_struct *task;
 	u32 status;
 
@@ -947,6 +950,7 @@ static __always_inline bool bpf_in_ia32_syscall()
 #endif
 
 	return status & TS_COMPAT;
+#endif // #ifdef __ppc64__
 }
 
 #endif
