@@ -22,6 +22,7 @@ or GPL2.txt for full copies of the license.
 #endif
 
 #include <linux/time.h>
+typedef u64 nanoseconds;
 
 /*
  * Global defines
@@ -39,6 +40,7 @@ or GPL2.txt for full copies of the license.
 #define PPM_PORT_MYSQL 3306
 #define PPM_PORT_POSTGRES 5432
 #define PPM_PORT_STATSD 8125
+#define PPM_PORT_MONGODB 27017
 
 /*
  * The ring descriptor.
@@ -50,7 +52,7 @@ struct ppm_ring_buffer_context {
 	bool capture_enabled;
 	struct ppm_ring_buffer_info *info;
 	char *buffer;
-	struct timespec last_print_time;
+	nanoseconds last_print_time;
 	u32 nevents;
 	atomic_t preempt_count;
 	char *str_storage;	/* String storage. Size is one page. */
@@ -116,4 +118,8 @@ extern const enum ppm_syscall_code g_syscall_ia32_code_routing_table[];
 
 extern void ppm_syscall_get_arguments(struct task_struct *task, struct pt_regs *regs, unsigned long *args);
 
+#define NS_TO_SEC(_ns) ((_ns) / 1000000000)
+#define MORE_THAN_ONE_SECOND_AHEAD(_ns1, _ns2) ((_ns1) - (_ns2) > 1000000000)
+#define SECOND_IN_NS 1000000000
+
 #endif /* PPM_H_ */
