@@ -1,19 +1,10 @@
 /*
-Copyright (C) 2013-2014 Draios inc.
 
-This file is part of sysdig.
+Copyright (c) 2013-2018 Draios Inc. dba Sysdig.
 
-sysdig is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License version 2 as
-published by the Free Software Foundation.
+This file is dual licensed under either the MIT or GPL 2. See MIT.txt
+or GPL2.txt for full copies of the license.
 
-sysdig is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #define pr_fmt(fmt)	KBUILD_MODNAME ": " fmt
@@ -28,7 +19,6 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <linux/tcp.h>
 #include <linux/udp.h>
 #include <linux/file.h>
-#include <linux/futex.h>
 #include <linux/fs_struct.h>
 #include <linux/uaccess.h>
 #include <linux/version.h>
@@ -47,7 +37,8 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include "ppm_events_public.h"
 #include "ppm_events.h"
 #include "ppm.h"
-
+#include "ppm_flag_helpers.h"
+#include "ppm_version.h"
 
 /*
  * The kernel patched with grsecurity makes the default access_ok trigger a
@@ -56,7 +47,11 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #ifdef access_ok_noprefault
 #define ppm_access_ok access_ok_noprefault
 #else
-#define ppm_access_ok access_ok
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)) || (PPM_RHEL_RELEASE_CODE > 0 && PPM_RHEL_RELEASE_CODE >= PPM_RHEL_RELEASE_VERSION(8, 1))
+#define ppm_access_ok(type, addr, size)	access_ok(addr, size)
+#else
+#define ppm_access_ok(type, addr, size)	access_ok(type, addr, size)
+#endif
 #endif
 
 extern bool g_tracers_enabled;
@@ -69,6 +64,11 @@ static void memory_dump(char *p, size_t size)
 		pr_info("%*ph\n", 8, &p[j]);
 }
 
+static inline bool in_port_range(uint16_t port, uint16_t min, uint16_t max)
+{
+	return port >= min && port <= max;
+}
+
 /*
  * Globals
  */
@@ -175,6 +175,41 @@ int32_t dpi_lookahead_init(void)
 	return PPM_SUCCESS;
 }
 
+inline int sock_getname(struct socket* sock, struct sockaddr* sock_address, int peer)
+{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 17, 0)
+	int ret = sock->ops->getname(sock, sock_address, peer);
+	if (ret >= 0)
+		ret = 0;
+	return ret;
+#else
+	int sockaddr_len;
+	return sock->ops->getname(sock, sock_address, &sockaddr_len, peer);
+#endif
+}
+
+/**
+ * Compute the snaplen for the arguments.
+ *
+ * The snaplen is the amount of argument data returned along with the event.
+ * Normally, the driver performs a dynamic calculation to figure out snaplen
+ * per-event. However, if this calculation is disabled
+ * (i.e. args->consumer->do_dynamic_snaplen == false), the snaplen will always
+ * be args->consumer->snaplen.
+ *
+ * If dynamic snaplen is enabled, here's how the calculation works:
+ *
+ * 1. If the event is a write to /dev/null, it gets a special snaplen because
+ *    writes to /dev/null is a backdoor method for inserting special events
+ *    into the event stream.
+ * 2. If the event is NOT a socket operation, return args->consumer->snaplen.
+ * 3. If the sending port OR destination port falls within the fullcapture port
+ *    range specified by the user, return 16000.
+ * 4. Protocol detection. A number of applications are detected heuristically
+ *    and given a longer snaplen (2000). These applications are MYSQL, Postgres,
+ *    HTTP, mongodb, and statsd.
+ * 5. If none of the above apply, return args->consumer->snaplen.
+ */
 inline u32 compute_snaplen(struct event_filler_arguments *args, char *buf, u32 lookahead_size)
 {
 	u32 res = args->consumer->snaplen;
@@ -183,10 +218,19 @@ inline u32 compute_snaplen(struct event_filler_arguments *args, char *buf, u32 l
 	sa_family_t family;
 	struct sockaddr_storage sock_address;
 	struct sockaddr_storage peer_address;
-	int sock_address_len;
-	int peer_address_len;
 	u16 sport, dport;
+	u16 min_port = 0, max_port = 0;
+	u32 dynamic_snaplen = 2000;
 
+	if (args->consumer->snaplen > dynamic_snaplen) {
+		/*
+		 * If the user requested a default snaplen greater than the custom
+		 * snaplen given to certain applications, just use the greater value.
+		 */
+		dynamic_snaplen = args->consumer->snaplen;
+	}
+
+	/* Increase snaplen on writes to /dev/null */
 	if (g_tracers_enabled && args->event_type == PPME_SYSCALL_WRITE_X) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 14, 0)
 		struct fd f = fdget(args->fd);
@@ -231,181 +275,216 @@ inline u32 compute_snaplen(struct event_filler_arguments *args, char *buf, u32 l
 
 	sock = sockfd_lookup(args->fd, &err);
 
-	if (sock) {
-
-		if (sock->sk) {
-			err = sock->ops->getname(sock, (struct sockaddr *)&sock_address, &sock_address_len, 0);
-
-			if (err == 0) {
-				if(args->event_type == PPME_SOCKET_SENDTO_X)
-				{
-					unsigned long val;
-					struct sockaddr __user * usrsockaddr;
-					/*
-					 * Get the address
-					 */
-					if (!args->is_socketcall)
-						syscall_get_arguments(current, args->regs, 4, 1, &val);
-					else
-						val = args->socketcall_args[4];
-
-					usrsockaddr = (struct sockaddr __user *)val;
-
-					if(usrsockaddr == NULL) {
-						/*
-						 * Suppose is a connected socket, fall back to fd
-						 */
-						err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
-					} else {
-						/*
-						 * Get the address len
-						 */
-						if (!args->is_socketcall)
-							syscall_get_arguments(current, args->regs, 5, 1, &val);
-						else
-							val = args->socketcall_args[5];
-
-						if (val != 0) {
-							peer_address_len = val;
-							/*
-							 * Copy the address
-							 */
-							err = addr_to_kernel(usrsockaddr, val, (struct sockaddr *)&peer_address);
-						} else {
-							/*
-							 * This case should be very rare, fallback again to sock
-							 */
-							err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
-						}
-					}
-				} else if (args->event_type == PPME_SOCKET_SENDMSG_X) {
-					unsigned long val;
-					struct sockaddr __user * usrsockaddr;
-					int addrlen;
-#ifdef CONFIG_COMPAT
-					struct compat_msghdr compat_mh;
-#endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 19, 0)
-					struct user_msghdr mh;
-#else
-					struct msghdr mh;
-#endif
-
-					if (!args->is_socketcall)
-						syscall_get_arguments(current, args->regs, 1, 1, &val);
-					else
-						val = args->socketcall_args[1];
-
-#ifdef CONFIG_COMPAT
-					if (!args->compat) {
-#endif
-						if (unlikely(ppm_copy_from_user(&mh, (const void __user *)val, sizeof(mh)))) {
-							usrsockaddr = NULL;
-							addrlen = 0;
-						} else {
-							usrsockaddr = (struct sockaddr __user *)mh.msg_name;
-							addrlen = mh.msg_namelen;
-						}
-#ifdef CONFIG_COMPAT
-					} else {
-						if (unlikely(ppm_copy_from_user(&compat_mh, (const void __user *)compat_ptr(val), sizeof(compat_mh)))) {
-							usrsockaddr = NULL;
-							addrlen = 0;
-						} else {
-							usrsockaddr = (struct sockaddr __user *)compat_ptr(compat_mh.msg_name);
-							addrlen = compat_mh.msg_namelen;
-						}
-					}
-#endif
-
-					if (usrsockaddr != NULL && addrlen != 0) {
-						peer_address_len = addrlen;
-						/*
-						 * Copy the address
-						 */
-						err = addr_to_kernel(usrsockaddr, peer_address_len, (struct sockaddr *)&peer_address);
-					} else
-						/*
-						 * Suppose it is a connected socket, fall back to fd
-						 */
-						err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
-				} else
-					err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
-
-				if (err == 0) {
-					family = sock->sk->sk_family;
-
-					if (family == AF_INET) {
-						sport = ntohs(((struct sockaddr_in *) &sock_address)->sin_port);
-						dport = ntohs(((struct sockaddr_in *) &peer_address)->sin_port);
-					} else if (family == AF_INET6) {
-						sport = ntohs(((struct sockaddr_in6 *) &sock_address)->sin6_port);
-						dport = ntohs(((struct sockaddr_in6 *) &peer_address)->sin6_port);
-					} else {
-						sport = 0;
-						dport = 0;
-					}
-
-					if (sport == PPM_PORT_MYSQL || dport == PPM_PORT_MYSQL) {
-						if (lookahead_size >= 5) {
-							if (buf[0] == 3 || buf[1] == 3 || buf[2] == 3 || buf[3] == 3 || buf[4] == 3) {
-								sockfd_put(sock);
-								return 2000;
-							} else if (buf[2] == 0 && buf[3] == 0) {
-								sockfd_put(sock);
-								return 2000;
-							}
-						}
-					} else if (sport == PPM_PORT_POSTGRES || dport == PPM_PORT_POSTGRES) {
-						if (lookahead_size >= 2) {
-							if ((buf[0] == 'Q' && buf[1] == 0) || /* SimpleQuery command */
-								(buf[0] == 'P' && buf[1] == 0) || /* Prepare statement commmand */
-								(buf[4] == 0 && buf[5] == 3 && buf[6] == 0) || /* startup command */
-								(buf[0] == 'E' && buf[1] == 0) /* error or execute command */
-							) {
-								sockfd_put(sock);
-								return 2000;
-							}
-						}
-					} else if ((lookahead_size >= 4 && buf[1] == 0 && buf[2] == 0 && buf[2] == 0) || /* matches command */
-								(lookahead_size >= 16 && (*(int32_t *)(buf+12) == 1 || /* matches header */
-									*(int32_t *)(buf+12) == 2001 ||
-									*(int32_t *)(buf+12) == 2002 ||
-									*(int32_t *)(buf+12) == 2003 ||
-									*(int32_t *)(buf+12) == 2004 ||
-									*(int32_t *)(buf+12) == 2005 ||
-									*(int32_t *)(buf+12) == 2006 ||
-									*(int32_t *)(buf+12) == 2007)
-							   )
-							) {
-						sockfd_put(sock);
-						return 2000;
-					} else if (dport == PPM_PORT_STATSD) {
-						sockfd_put(sock);
-						return 2000;
-					} else {
-						if (lookahead_size >= 5) {
-							if (*(u32 *)buf == g_http_get_intval ||
-								*(u32 *)buf == g_http_post_intval ||
-								*(u32 *)buf == g_http_put_intval ||
-								*(u32 *)buf == g_http_delete_intval ||
-								*(u32 *)buf == g_http_trace_intval ||
-								*(u32 *)buf == g_http_connect_intval ||
-								*(u32 *)buf == g_http_options_intval ||
-								((*(u32 *)buf == g_http_resp_intval) && (buf[4] == '/'))
-							) {
-								sockfd_put(sock);
-								return 2000;
-							}
-						}
-					}
-				}
-			}
-		}
-
-		sockfd_put(sock);
+	if (!sock) {
+		return res;
 	}
 
+	if (!sock->sk) {
+		goto done;
+	}
+
+	err = sock_getname(sock, (struct sockaddr *)&sock_address, 0);
+
+	if (err != 0) {
+		goto done;
+	}
+
+	/* Try to get the source and destination port */
+	if (args->event_type == PPME_SOCKET_SENDTO_X) {
+		unsigned long syscall_args[6] = {};
+		unsigned long val;
+		struct sockaddr __user * usrsockaddr;
+		/*
+		 * Get the address
+		 */
+		if (!args->is_socketcall) {
+			ppm_syscall_get_arguments(current, args->regs, syscall_args);
+			val = syscall_args[4];
+		} else {
+			val = args->socketcall_args[4];
+		}
+
+		usrsockaddr = (struct sockaddr __user *)val;
+
+		if(usrsockaddr == NULL) {
+			/*
+			 * Suppose is a connected socket, fall back to fd
+			 */
+			err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
+		} else {
+			/*
+			 * Get the address len
+			 */
+			if (!args->is_socketcall) {
+				ppm_syscall_get_arguments(current, args->regs, syscall_args);
+				val = syscall_args[5];
+			} else {
+				val = args->socketcall_args[5];
+			}
+
+			if (val != 0) {
+				/*
+				 * Copy the address
+				 */
+				err = addr_to_kernel(usrsockaddr, val, (struct sockaddr *)&peer_address);
+			} else {
+				/*
+				 * This case should be very rare, fallback again to sock
+				 */
+				err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
+			}
+		}
+	} else if (args->event_type == PPME_SOCKET_SENDMSG_X) {
+		unsigned long syscall_args[6] = {};
+		unsigned long val;
+		struct sockaddr __user * usrsockaddr;
+		int addrlen;
+#ifdef CONFIG_COMPAT
+		struct compat_msghdr compat_mh;
+#endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 19, 0)
+		struct user_msghdr mh;
+#else
+		struct msghdr mh;
+#endif
+
+		if (!args->is_socketcall) {
+			ppm_syscall_get_arguments(current, args->regs, syscall_args);
+			val = syscall_args[1];
+		} else {
+			val = args->socketcall_args[1];
+		}
+
+#ifdef CONFIG_COMPAT
+		if (!args->compat) {
+#endif
+			if (unlikely(ppm_copy_from_user(&mh, (const void __user *)val, sizeof(mh)))) {
+				usrsockaddr = NULL;
+				addrlen = 0;
+			} else {
+				usrsockaddr = (struct sockaddr __user *)mh.msg_name;
+				addrlen = mh.msg_namelen;
+			}
+#ifdef CONFIG_COMPAT
+		} else {
+			if (unlikely(ppm_copy_from_user(&compat_mh, (const void __user *)compat_ptr(val), sizeof(compat_mh)))) {
+				usrsockaddr = NULL;
+				addrlen = 0;
+			} else {
+				usrsockaddr = (struct sockaddr __user *)compat_ptr(compat_mh.msg_name);
+				addrlen = compat_mh.msg_namelen;
+			}
+		}
+#endif
+
+		if (usrsockaddr != NULL && addrlen != 0) {
+			/*
+			 * Copy the address
+			 */
+			err = addr_to_kernel(usrsockaddr, addrlen, (struct sockaddr *)&peer_address);
+		} else {
+			/*
+			 * Suppose it is a connected socket, fall back to fd
+			 */
+			err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
+		}
+	} else {
+		err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
+	}
+
+	if (err != 0) {
+		goto done;
+	}
+
+	/*
+	 * If there's a valid source / dest port, use it to run heuristics
+	 * for determining snaplen.
+	 */
+	min_port = args->consumer->fullcapture_port_range_start;
+	max_port = args->consumer->fullcapture_port_range_end;
+	family = sock->sk->sk_family;
+
+	if (family == AF_INET) {
+		sport = ntohs(((struct sockaddr_in *) &sock_address)->sin_port);
+		dport = ntohs(((struct sockaddr_in *) &peer_address)->sin_port);
+	} else if (family == AF_INET6) {
+		sport = ntohs(((struct sockaddr_in6 *) &sock_address)->sin6_port);
+		dport = ntohs(((struct sockaddr_in6 *) &peer_address)->sin6_port);
+	} else {
+		sport = 0;
+		dport = 0;
+	}
+
+	if (max_port > 0 &&
+	    (in_port_range(sport, min_port, max_port) ||
+	     in_port_range(dport, min_port, max_port))) {
+		/*
+		 * Before checking the well-known ports, see if the user has requested
+		 * an increased snaplen for the port in question.
+		 */
+		sockfd_put(sock);
+		return RW_MAX_FULLCAPTURE_PORT_SNAPLEN;
+	} else if (sport == PPM_PORT_MYSQL || dport == PPM_PORT_MYSQL) {
+		if (lookahead_size >= 5) {
+			if (buf[0] == 3 || buf[1] == 3 || buf[2] == 3 || buf[3] == 3 || buf[4] == 3) {
+				res = dynamic_snaplen;
+				goto done;
+			} else if (buf[2] == 0 && buf[3] == 0) {
+				res = dynamic_snaplen;
+				goto done;
+			}
+		}
+	} else if (sport == PPM_PORT_POSTGRES || dport == PPM_PORT_POSTGRES) {
+		if (lookahead_size >= 2) {
+			if ((buf[0] == 'Q' && buf[1] == 0) || /* SimpleQuery command */
+			    (buf[0] == 'P' && buf[1] == 0) || /* Prepare statement command */
+			    (buf[0] == 'E' && buf[1] == 0) /* error or execute command */
+			) {
+				res = dynamic_snaplen;
+				goto done;
+			}
+		}
+		if (lookahead_size >= 7 &&
+		    (buf[4] == 0 && buf[5] == 3 && buf[6] == 0)) { /* startup command */
+			res = dynamic_snaplen;
+			goto done;
+		}
+	} else if ((sport == PPM_PORT_MONGODB || dport == PPM_PORT_MONGODB) ||
+	            (lookahead_size >= 16 &&
+	               (*(int32_t *)(buf+12) == 1    || /* matches header */
+	                *(int32_t *)(buf+12) == 2001 ||
+	                *(int32_t *)(buf+12) == 2002 ||
+	                *(int32_t *)(buf+12) == 2003 ||
+	                *(int32_t *)(buf+12) == 2004 ||
+	                *(int32_t *)(buf+12) == 2005 ||
+	                *(int32_t *)(buf+12) == 2006 ||
+	                *(int32_t *)(buf+12) == 2007)
+	            )
+	          ) {
+		res = dynamic_snaplen;
+		goto done;
+	} else if (dport == args->consumer->statsd_port) {
+		res = dynamic_snaplen;
+		goto done;
+	} else {
+		if (lookahead_size >= 5) {
+			if (*(u32 *)buf == g_http_get_intval ||
+			    *(u32 *)buf == g_http_post_intval ||
+			    *(u32 *)buf == g_http_put_intval ||
+			    *(u32 *)buf == g_http_delete_intval ||
+			    *(u32 *)buf == g_http_trace_intval ||
+			    *(u32 *)buf == g_http_connect_intval ||
+			    *(u32 *)buf == g_http_options_intval ||
+			    ((*(u32 *)buf == g_http_resp_intval) && (buf[4] == '/'))
+			) {
+				res = dynamic_snaplen;
+				goto done;
+			}
+		}
+	}
+
+done:
+	sockfd_put(sock);
 	return res;
 }
 
@@ -415,11 +494,12 @@ inline u32 compute_snaplen(struct event_filler_arguments *args, char *buf, u32 l
  * - fromuser is ignored for numeric types
  * - dyn_idx is ignored for everything other than PT_DYN
  */
-int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len, bool fromuser, u8 dyn_idx)
+int val_to_ring(struct event_filler_arguments *args, uint64_t val, u32 val_len, bool fromuser, u8 dyn_idx)
 {
 	const struct ppm_param_info *param_info;
 	int len = -1;
 	u16 *psize = (u16 *)(args->buffer + args->curarg * sizeof(u16));
+	u32 max_arg_size = args->arg_data_size;
 
 	if (unlikely(args->curarg >= args->nargs)) {
 		pr_err("(%u)val_to_ring: too many arguments for event #%u, type=%u, curarg=%u, nargs=%u tid:%u\n",
@@ -437,6 +517,9 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	if (unlikely(args->arg_data_size == 0))
 		return PPM_FAILURE_BUFFER_FULL;
 
+	if (max_arg_size > PPM_MAX_ARG_SIZE)
+		max_arg_size = PPM_MAX_ARG_SIZE;
+
 	param_info = &(g_event_info[args->event_type].params[args->curarg]);
 	if (param_info->type == PT_DYN && param_info->info != NULL) {
 		const struct ppm_param_info *dyn_params;
@@ -449,7 +532,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 		dyn_params = (const struct ppm_param_info *)param_info->info;
 
 		param_info = &dyn_params[dyn_idx];
-		if (likely(args->arg_data_size >= sizeof(u8)))	{
+		if (likely(max_arg_size >= sizeof(u8)))	{
 			*(u8 *)(args->buffer + args->arg_data_offset) = dyn_idx;
 			len = sizeof(u8);
 		} else {
@@ -457,6 +540,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 		}
 		args->arg_data_offset += len;
 		args->arg_data_size -= len;
+		max_arg_size -= len;
 		*psize = (u16)len;
 	} else {
 		*psize = 0;
@@ -468,17 +552,17 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 		if (likely(val != 0)) {
 			if (fromuser) {
 				len = ppm_strncpy_from_user(args->buffer + args->arg_data_offset,
-					(const char __user *)(unsigned long)val, args->arg_data_size);
+					(const char __user *)(unsigned long)val, max_arg_size);
 
 				if (unlikely(len < 0))
 					return PPM_FAILURE_INVALID_USER_MEMORY;
 			} else {
 				len = strlcpy(args->buffer + args->arg_data_offset,
 								(const char *)(unsigned long)val,
-								args->arg_data_size);
+								max_arg_size);
 
-				if (++len > args->arg_data_size)
-					len = args->arg_data_size;
+				if (++len > max_arg_size)
+					len = max_arg_size;
 			}
 
 			/*
@@ -491,10 +575,10 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 			 */
 			len = strlcpy(args->buffer + args->arg_data_offset,
 				"(NULL)",
-				args->arg_data_size);
+				max_arg_size);
 
-			if (++len > args->arg_data_size)
-				len = args->arg_data_size;
+			if (++len > max_arg_size)
+				len = max_arg_size;
 		}
 
 		break;
@@ -505,12 +589,12 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 				 * Copy the lookahead portion of the buffer that we will use DPI-based
 				 * snaplen calculation
 				 */
-				u32 dpi_lookahead_size = DPI_LOOKAHED_SIZE;
+				u32 dpi_lookahead_size = DPI_LOOKAHEAD_SIZE;
 
 				if (dpi_lookahead_size > val_len)
 					dpi_lookahead_size = val_len;
 
-				if (unlikely(dpi_lookahead_size >= args->arg_data_size))
+				if (unlikely(dpi_lookahead_size >= max_arg_size))
 					return PPM_FAILURE_BUFFER_FULL;
 
 				len = (int)ppm_copy_from_user(args->buffer + args->arg_data_offset,
@@ -536,8 +620,8 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 							val_len = sl;
 					}
 
-					if (unlikely((val_len) >= args->arg_data_size))
-						val_len = args->arg_data_size;
+					if (unlikely((val_len) >= max_arg_size))
+						val_len = max_arg_size;
 
 					if (val_len > dpi_lookahead_size) {
 						len = (int)ppm_copy_from_user(args->buffer + args->arg_data_offset + dpi_lookahead_size,
@@ -558,7 +642,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 						val_len = sl;
 				}
 
-				if (unlikely(val_len >= args->arg_data_size))
+				if (unlikely(val_len >= max_arg_size))
 					return PPM_FAILURE_BUFFER_FULL;
 
 				memcpy(args->buffer + args->arg_data_offset,
@@ -578,7 +662,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	case PT_SOCKTUPLE:
 	case PT_FDLIST:
 		if (likely(val != 0)) {
-			if (unlikely(val_len >= args->arg_data_size))
+			if (unlikely(val_len >= max_arg_size))
 				return PPM_FAILURE_BUFFER_FULL;
 
 			if (fromuser) {
@@ -607,7 +691,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	case PT_FLAGS8:
 	case PT_UINT8:
 	case PT_SIGTYPE:
-		if (likely(args->arg_data_size >= sizeof(u8)))	{
+		if (likely(max_arg_size >= sizeof(u8)))	{
 			*(u8 *)(args->buffer + args->arg_data_offset) = (u8)val;
 			len = sizeof(u8);
 		} else {
@@ -618,7 +702,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	case PT_FLAGS16:
 	case PT_UINT16:
 	case PT_SYSCALLID:
-		if (likely(args->arg_data_size >= sizeof(u16))) {
+		if (likely(max_arg_size >= sizeof(u16))) {
 			*(u16 *)(args->buffer + args->arg_data_offset) = (u16)val;
 			len = sizeof(u16);
 		} else {
@@ -628,10 +712,11 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 		break;
 	case PT_FLAGS32:
 	case PT_UINT32:
+	case PT_MODE:
 	case PT_UID:
 	case PT_GID:
 	case PT_SIGSET:
-		if (likely(args->arg_data_size >= sizeof(u32))) {
+		if (likely(max_arg_size >= sizeof(u32))) {
 			*(u32 *)(args->buffer + args->arg_data_offset) = (u32)val;
 			len = sizeof(u32);
 		} else {
@@ -642,7 +727,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	case PT_RELTIME:
 	case PT_ABSTIME:
 	case PT_UINT64:
-		if (likely(args->arg_data_size >= sizeof(u64))) {
+		if (likely(max_arg_size >= sizeof(u64))) {
 			*(u64 *)(args->buffer + args->arg_data_offset) = (u64)val;
 			len = sizeof(u64);
 		} else {
@@ -651,7 +736,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 
 		break;
 	case PT_INT8:
-		if (likely(args->arg_data_size >= sizeof(s8))) {
+		if (likely(max_arg_size >= sizeof(s8))) {
 			*(s8 *)(args->buffer + args->arg_data_offset) = (s8)(long)val;
 			len = sizeof(s8);
 		} else {
@@ -660,7 +745,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 
 		break;
 	case PT_INT16:
-		if (likely(args->arg_data_size >= sizeof(s16))) {
+		if (likely(max_arg_size >= sizeof(s16))) {
 			*(s16 *)(args->buffer + args->arg_data_offset) = (s16)(long)val;
 			len = sizeof(s16);
 		} else {
@@ -669,7 +754,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 
 		break;
 	case PT_INT32:
-		if (likely(args->arg_data_size >= sizeof(s32))) {
+		if (likely(max_arg_size >= sizeof(s32))) {
 			*(s32 *)(args->buffer + args->arg_data_offset) = (s32)(long)val;
 			len = sizeof(s32);
 		} else {
@@ -681,7 +766,7 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	case PT_ERRNO:
 	case PT_FD:
 	case PT_PID:
-		if (likely(args->arg_data_size >= sizeof(s64))) {
+		if (likely(max_arg_size >= sizeof(s64))) {
 			*(s64 *)(args->buffer + args->arg_data_offset) = (s64)(long)val;
 			len = sizeof(s64);
 		} else {
@@ -698,8 +783,8 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 		return PPM_FAILURE_BUG;
 	}
 
-	ASSERT(len <= 65535);
-	ASSERT(len <= args->arg_data_size);
+	ASSERT(len <= PPM_MAX_ARG_SIZE);
+	ASSERT(len <= max_arg_size);
 
 	*psize += (u16)len;
 	args->curarg++;
@@ -709,110 +794,6 @@ int val_to_ring(struct event_filler_arguments *args, uint64_t val, u16 val_len,
 	return PPM_SUCCESS;
 }
 
-static inline u8 socket_family_to_scap(u8 family)
-{
-	if (family == AF_INET)
-		return PPM_AF_INET;
-	else if (family == AF_INET6)
-		return PPM_AF_INET6;
-	else if (family == AF_UNIX)
-		return PPM_AF_UNIX;
-	else if (family == AF_NETLINK)
-		return PPM_AF_NETLINK;
-	else if (family == AF_PACKET)
-		return PPM_AF_PACKET;
-	else if (family == AF_UNSPEC)
-		return PPM_AF_UNSPEC;
-	else if (family == AF_AX25)
-		return PPM_AF_AX25;
-	else if (family == AF_IPX)
-		return PPM_AF_IPX;
-	else if (family == AF_APPLETALK)
-		return PPM_AF_APPLETALK;
-	else if (family == AF_NETROM)
-		return PPM_AF_NETROM;
-	else if (family == AF_BRIDGE)
-		return PPM_AF_BRIDGE;
-	else if (family == AF_ATMPVC)
-		return PPM_AF_ATMPVC;
-	else if (family == AF_X25)
-		return PPM_AF_X25;
-	else if (family == AF_ROSE)
-		return PPM_AF_ROSE;
-	else if (family == AF_DECnet)
-		return PPM_AF_DECnet;
-	else if (family == AF_NETBEUI)
-		return PPM_AF_NETBEUI;
-	else if (family == AF_SECURITY)
-		return PPM_AF_SECURITY;
-	else if (family == AF_KEY)
-		return PPM_AF_KEY;
-	else if (family == AF_ROUTE)
-		return PPM_AF_ROUTE;
-	else if (family == AF_ASH)
-		return PPM_AF_ASH;
-	else if (family == AF_ECONET)
-		return PPM_AF_ECONET;
-	else if (family == AF_ATMSVC)
-		return PPM_AF_ATMSVC;
-#ifdef AF_RDS
-	else if (family == AF_RDS)
-		return PPM_AF_RDS;
-#endif
-	else if (family == AF_SNA)
-		return PPM_AF_SNA;
-	else if (family == AF_IRDA)
-		return PPM_AF_IRDA;
-	else if (family == AF_PPPOX)
-		return PPM_AF_PPPOX;
-	else if (family == AF_WANPIPE)
-		return PPM_AF_WANPIPE;
-	else if (family == AF_LLC)
-		return PPM_AF_LLC;
-#ifdef AF_CAN
-	else if (family == AF_CAN)
-		return PPM_AF_CAN;
-#endif
-	 else if (family == AF_TIPC)
-		return PPM_AF_TIPC;
-	else if (family == AF_BLUETOOTH)
-		return PPM_AF_BLUETOOTH;
-	else if (family == AF_IUCV)
-		return PPM_AF_IUCV;
-#ifdef AF_RXRPC
-	else if (family == AF_RXRPC)
-		return PPM_AF_RXRPC;
-#endif
-#ifdef AF_ISDN
-	else if (family == AF_ISDN)
-		return PPM_AF_ISDN;
-#endif
-#ifdef AF_PHONET
-	else if (family == AF_PHONET)
-		return PPM_AF_PHONET;
-#endif
-#ifdef AF_IEEE802154
-	else if (family == AF_IEEE802154)
-		return PPM_AF_IEEE802154;
-#endif
-#ifdef AF_CAIF
-	else if (family == AF_CAIF)
-		return PPM_AF_CAIF;
-#endif
-#ifdef AF_ALG
-	else if (family == AF_ALG)
-		return PPM_AF_ALG;
-#endif
-#ifdef AF_NFC
-	else if (family == AF_NFC)
-		return PPM_AF_NFC;
-#endif
-	else {
-		ASSERT(false);
-		return PPM_AF_UNSPEC;
-	}
-}
-
 /*
 static struct socket *ppm_sockfd_lookup_light(int fd, int *err, int *fput_needed)
 {
@@ -963,8 +944,6 @@ u16 fd_to_socktuple(int fd,
 	char *dest;
 	struct sockaddr_storage sock_address;
 	struct sockaddr_storage peer_address;
-	int sock_address_len;
-	int peer_address_len;
 
 	/*
 	 * Get the socket from the fd
@@ -982,7 +961,7 @@ u16 fd_to_socktuple(int fd,
 		return 0;
 	}
 
-	err = sock->ops->getname(sock, (struct sockaddr *)&sock_address, &sock_address_len, 0);
+	err = sock_getname(sock, (struct sockaddr *)&sock_address, 0);
 	ASSERT(err == 0);
 
 	family = sock->sk->sk_family;
@@ -993,7 +972,7 @@ u16 fd_to_socktuple(int fd,
 	switch (family) {
 	case AF_INET:
 		if (!use_userdata) {
-			err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
+			err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
 			if (err == 0) {
 				if (is_inbound) {
 					sip = ((struct sockaddr_in *) &peer_address)->sin_addr.s_addr;
@@ -1045,7 +1024,7 @@ u16 fd_to_socktuple(int fd,
 		break;
 	case AF_INET6:
 		if (!use_userdata) {
-			err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
+			err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
 			ASSERT(err == 0);
 
 			if (is_inbound) {
@@ -1120,7 +1099,7 @@ u16 fd_to_socktuple(int fd,
 			if (is_inbound) {
 				us_name = ((struct sockaddr_un *) &sock_address)->sun_path;
 			} else {
-				err = sock->ops->getname(sock, (struct sockaddr *)&peer_address, &peer_address_len, 1);
+				err = sock_getname(sock, (struct sockaddr *)&peer_address, 1);
 				ASSERT(err == 0);
 
 				us_name = ((struct sockaddr_un *) &peer_address)->sun_path;
@@ -1189,18 +1168,22 @@ int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struc
 {
 	int32_t res;
 	const struct iovec *iov;
-	u32 copylen;
+	u64 copylen;
 	u32 j;
 	u64 size = 0;
 	unsigned long bufsize;
 	char *targetbuf = args->str_storage;
 	u32 targetbuflen = STR_STORAGE_SIZE;
+	unsigned long syscall_args[6] = {};
 	unsigned long val;
 	u32 notcopied_len;
 	size_t tocopy_len;
 
 	copylen = iovcnt * sizeof(struct iovec);
 
+	if (unlikely(iovcnt >= 0xffffffff))
+		return PPM_FAILURE_BUFFER_FULL;
+
 	if (unlikely(copylen >= STR_STORAGE_SIZE))
 		return PPM_FAILURE_BUFFER_FULL;
 
@@ -1240,9 +1223,10 @@ int32_t parse_readv_writev_bufs(struct event_filler_arguments *args, const struc
 			/*
 			 * Retrieve the FD. It will be used for dynamic snaplen calculation.
 			 */
-			if (!args->is_socketcall)
-				syscall_get_arguments(current, args->regs, 0, 1, &val);
-			else
+			if (!args->is_socketcall) {
+				ppm_syscall_get_arguments(current, args->regs, syscall_args);
+				val = syscall_args[0];
+			} else
 				val = args->socketcall_args[0];
 			args->fd = (int)val;
 
@@ -1320,18 +1304,22 @@ int32_t compat_parse_readv_writev_bufs(struct event_filler_arguments *args, cons
 {
 	int32_t res;
 	const struct compat_iovec *iov;
-	u32 copylen;
+	u64 copylen;
 	u32 j;
 	u64 size = 0;
 	unsigned long bufsize;
 	char *targetbuf = args->str_storage;
 	u32 targetbuflen = STR_STORAGE_SIZE;
+	unsigned long syscall_args[6] = {};
 	unsigned long val;
 	u32 notcopied_len;
 	compat_size_t tocopy_len;
 
 	copylen = iovcnt * sizeof(struct compat_iovec);
 
+	if (unlikely(iovcnt >= 0xffffffff))
+		return PPM_FAILURE_BUFFER_FULL;
+
 	if (unlikely(copylen >= STR_STORAGE_SIZE))
 		return PPM_FAILURE_BUFFER_FULL;
 
@@ -1371,9 +1359,10 @@ int32_t compat_parse_readv_writev_bufs(struct event_filler_arguments *args, cons
 			/*
 			 * Retrieve the FD. It will be used for dynamic snaplen calculation.
 			 */
-			if (!args->is_socketcall)
-				syscall_get_arguments(current, args->regs, 0, 1, &val);
-			else
+			if (!args->is_socketcall) {
+				ppm_syscall_get_arguments(current, args->regs, syscall_args);
+				val = syscall_args[0];
+			} else
 				val = args->socketcall_args[0];
 			args->fd = (int)val;
 
@@ -1454,13 +1443,15 @@ int32_t compat_parse_readv_writev_bufs(struct event_filler_arguments *args, cons
  * filler function.
  * The arguments to extract are be specified in g_ppm_events.
  */
-int f_sys_autofill(struct event_filler_arguments *args, const struct ppm_event_entry *evinfo)
+int f_sys_autofill(struct event_filler_arguments *args)
 {
 	int res;
+	unsigned long syscall_args[6] = {};
 	unsigned long val;
 	u32 j;
 	int64_t retval;
 
+	const struct ppm_event_entry *evinfo = &g_ppm_events[args->event_type];
 	ASSERT(evinfo->n_autofill_args <= PPM_MAX_AUTOFILL_ARGS);
 
 	for (j = 0; j < evinfo->n_autofill_args; j++) {
@@ -1474,11 +1465,8 @@ int f_sys_autofill(struct event_filler_arguments *args, const struct ppm_event_e
 				/*
 				 * Regular argument
 				 */
-				syscall_get_arguments(current,
-						args->regs,
-						evinfo->autofill_args[j].id,
-						1,
-						&val);
+				ppm_syscall_get_arguments(current, args->regs, syscall_args);
+				val = syscall_args[evinfo->autofill_args[j].id];
 			}
 
 			res = val_to_ring(args, val, 0, true, 0);
