@@ -19,7 +19,9 @@ limitations under the License.
 
 #include <algorithm>
 
+#if defined(HAS_CAPTURE)
 #include "container_engine/cri.h"
+#endif
 #include "container_engine/docker.h"
 #include "container_engine/rkt.h"
 #include "container_engine/libvirt_lxc.h"
@@ -74,15 +76,17 @@ bool sinsp_container_manager::remove_inactive_containers()
 			return true;
 		});
 
-		for(unordered_map<string, sinsp_container_info>::iterator it = m_containers.begin(); it != m_containers.end();)
+		auto containers = m_containers.lock();
+		for(auto it = containers->begin(); it != containers->end();)
 		{
 			if(containers_in_use.find(it->first) == containers_in_use.end())
 			{
+				sinsp_container_info::ptr_t container = it->second;
 				for(const auto &remove_cb : m_remove_callbacks)
 				{
-					remove_cb(m_containers[it->first]);
+					remove_cb(*container);
 				}
-				m_containers.erase(it++);
+				containers->erase(it++);
 			}
 			else
 			{
@@ -94,12 +98,13 @@ bool sinsp_container_manager::remove_inactive_containers()
 	return res;
 }
 
-sinsp_container_info* sinsp_container_manager::get_container(const string& container_id)
+sinsp_container_info::ptr_t sinsp_container_manager::get_container(const string& container_id) const
 {
-	auto it = m_containers.find(container_id);
-	if(it != m_containers.end())
+	auto containers = m_containers.lock();
+	auto it = containers->find(container_id);
+	if(it != containers->end())
 	{
-		return &it->second;
+		return it->second;
 	}
 
 	return nullptr;
@@ -125,7 +130,7 @@ bool sinsp_container_manager::resolve_container(sinsp_threadinfo* tinfo, bool qu
 
 	for(auto &eng : m_container_engines)
 	{
-		matches = matches || eng->resolve(this, tinfo, query_os_for_missing_info);
+		matches = matches || eng->resolve(tinfo, query_os_for_missing_info);
 
 		if(matches)
 		{
@@ -144,6 +149,7 @@ string sinsp_container_manager::container_to_json(const sinsp_container_info& co
 	Json::Value obj;
 	Json::Value& container = obj["container"];
 	container["id"] = container_info.m_id;
+	container["full_id"] = container_info.m_full_id;
 	container["type"] = container_info.m_type;
 	container["name"] = container_info.m_name;
 	container["image"] = container_info.m_image;
@@ -153,6 +159,7 @@ string sinsp_container_manager::container_to_json(const sinsp_container_info& co
 	container["imagedigest"] = container_info.m_imagedigest;
 	container["privileged"] = container_info.m_privileged;
 	container["is_pod_sandbox"] = container_info.m_is_pod_sandbox;
+	container["lookup_state"] = static_cast<int>(container_info.m_lookup_state);
 
 	Json::Value mounts = Json::arrayValue;
 
@@ -273,21 +280,32 @@ bool sinsp_container_manager::container_to_sinsp_event(const string& json, sinsp
 	return true;
 }
 
-const unordered_map<string, sinsp_container_info>* sinsp_container_manager::get_containers()
+sinsp_container_manager::map_ptr_t sinsp_container_manager::get_containers() const
 {
-	return &m_containers;
+	return m_containers.lock();
 }
 
-void sinsp_container_manager::add_container(const sinsp_container_info& container_info, sinsp_threadinfo *thread_info)
+void sinsp_container_manager::add_container(const sinsp_container_info::ptr_t& container_info, sinsp_threadinfo *thread)
 {
-	m_containers[container_info.m_id] = container_info;
+	set_lookup_status(container_info->m_id, container_info->m_type, container_info->m_lookup_state);
+	{
+		auto containers = m_containers.lock();
+		(*containers)[container_info->m_id] = container_info;
+	}
 
 	for(const auto &new_cb : m_new_callbacks)
 	{
-		new_cb(m_containers[container_info.m_id], thread_info);
+		new_cb(*container_info, thread);
 	}
 }
 
+void sinsp_container_manager::replace_container(const sinsp_container_info::ptr_t& container_info)
+{
+	auto containers = m_containers.lock();
+	ASSERT(containers->find(container_info->m_id) != containers->end());
+	(*containers)[container_info->m_id] = container_info;
+}
+
 void sinsp_container_manager::notify_new_container(const sinsp_container_info& container_info)
 {
 	sinsp_evt *evt = new sinsp_evt();
@@ -301,7 +319,9 @@ void sinsp_container_manager::notify_new_container(const sinsp_container_info& c
 		std::shared_ptr<sinsp_evt> cevt(evt);
 
 		// Enqueue it onto the queue of pending container events for the inspector
+#ifndef _WIN32
 		m_inspector->m_pending_container_evts.push(cevt);
+#endif
 	}
 	else
 	{
@@ -314,10 +334,10 @@ void sinsp_container_manager::notify_new_container(const sinsp_container_info& c
 
 void sinsp_container_manager::dump_containers(scap_dumper_t* dumper)
 {
-	for(unordered_map<string, sinsp_container_info>::const_iterator it = m_containers.begin(); it != m_containers.end(); ++it)
+	for(const auto& it : (*m_containers.lock()))
 	{
 		sinsp_evt evt;
-		if(container_to_sinsp_event(container_to_json(it->second), &evt, it->second.get_tinfo(m_inspector)))
+		if(container_to_sinsp_event(container_to_json(*it.second), &evt, it.second->get_tinfo(m_inspector)))
 		{
 			int32_t res = scap_dump(m_inspector->m_h, dumper, evt.m_pevt, evt.m_cpuid, 0);
 			if(res != SCAP_SUCCESS)
@@ -328,7 +348,7 @@ void sinsp_container_manager::dump_containers(scap_dumper_t* dumper)
 	}
 }
 
-string sinsp_container_manager::get_container_name(sinsp_threadinfo* tinfo)
+string sinsp_container_manager::get_container_name(sinsp_threadinfo* tinfo) const
 {
 	string res;
 
@@ -338,7 +358,7 @@ string sinsp_container_manager::get_container_name(sinsp_threadinfo* tinfo)
 	}
 	else
 	{
-		const sinsp_container_info *container_info = get_container(tinfo->m_container_id);
+		const sinsp_container_info::ptr_t container_info = get_container(tinfo->m_container_id);
 
 		if(!container_info)
 		{
@@ -363,13 +383,6 @@ void sinsp_container_manager::identify_category(sinsp_threadinfo *tinfo)
 		return;
 	}
 
-	sinsp_container_info *cinfo = get_container(tinfo->m_container_id);
-
-	if(!cinfo)
-	{
-		return;
-	}
-
 	if(tinfo->m_vpid == 1)
 	{
 		if(g_logger.get_severity() >= sinsp_logger::SEV_DEBUG)
@@ -385,7 +398,7 @@ void sinsp_container_manager::identify_category(sinsp_threadinfo *tinfo)
 	}
 
 	// Categories are passed from parent to child threads
-	sinsp_threadinfo* ptinfo = tinfo->get_parent_thread();
+	const sinsp_threadinfo* ptinfo = tinfo->get_parent_thread();
 
 	if(ptinfo && ptinfo->m_category != sinsp_threadinfo::CAT_NONE)
 	{
@@ -400,7 +413,13 @@ void sinsp_container_manager::identify_category(sinsp_threadinfo *tinfo)
 		return;
 	}
 
-	if(!cinfo->m_metadata_complete)
+	sinsp_container_info::ptr_t cinfo = get_container(tinfo->m_container_id);
+	if(!cinfo)
+	{
+		return;
+	}
+
+	if(!cinfo->is_successful())
 	{
 		if(g_logger.get_severity() >= sinsp_logger::SEV_DEBUG)
 		{
@@ -486,17 +505,75 @@ void sinsp_container_manager::subscribe_on_remove_container(remove_container_cb
 
 void sinsp_container_manager::create_engines()
 {
-	m_container_engines.emplace_back(new container_engine::docker());
-#ifndef CYGWING_AGENT
+#ifdef CYGWING_AGENT
+	{
+		auto docker_engine = std::make_shared<container_engine::docker>(*this, m_inspector /*wmi source*/);
+		m_container_engines.push_back(docker_engine);
+		m_container_engine_by_type[CT_DOCKER] = docker_engine;
+	}
+#else
+#ifndef _WIN32
+	{
+		auto docker_engine = std::make_shared<container_engine::docker>(*this);
+		m_container_engines.push_back(docker_engine);
+		m_container_engine_by_type[CT_DOCKER] = docker_engine;
+	}
+
 #if defined(HAS_CAPTURE)
-	m_container_engines.emplace_back(new container_engine::cri());
-#endif
-	m_container_engines.emplace_back(new container_engine::lxc());
-	m_container_engines.emplace_back(new container_engine::libvirt_lxc());
-	m_container_engines.emplace_back(new container_engine::mesos());
-	m_container_engines.emplace_back(new container_engine::rkt());
-	m_container_engines.emplace_back(new container_engine::bpm());
+	{
+		auto cri_engine = std::make_shared<container_engine::cri>(*this);
+		m_container_engines.push_back(cri_engine);
+		m_container_engine_by_type[CT_CRI] = cri_engine;
+		m_container_engine_by_type[CT_CRIO] = cri_engine;
+		m_container_engine_by_type[CT_CONTAINERD] = cri_engine;
+	}
 #endif
+	{
+		auto lxc_engine = std::make_shared<container_engine::lxc>(*this);
+		m_container_engines.push_back(lxc_engine);
+		m_container_engine_by_type[CT_LXC] = lxc_engine;
+	}
+	{
+		auto libvirt_lxc_engine = std::make_shared<container_engine::libvirt_lxc>(*this);
+		m_container_engines.push_back(libvirt_lxc_engine);
+		m_container_engine_by_type[CT_LIBVIRT_LXC] = libvirt_lxc_engine;
+	}
+
+	{
+		auto mesos_engine = std::make_shared<container_engine::mesos>(*this);
+		m_container_engines.push_back(mesos_engine);
+		m_container_engine_by_type[CT_MESOS] = mesos_engine;
+	}
+	{
+		auto rkt_engine = std::make_shared<container_engine::rkt>(*this);
+		m_container_engines.push_back(rkt_engine);
+		m_container_engine_by_type[CT_RKT] = rkt_engine;
+	}
+	{
+		auto bpm_engine = std::make_shared<container_engine::bpm>(*this);
+		m_container_engines.push_back(bpm_engine);
+		m_container_engine_by_type[CT_BPM] = bpm_engine;
+	}
+#endif // _WIN32
+#endif // CYGWING_AGENT
+}
+
+void sinsp_container_manager::update_container_with_size(sinsp_container_type type,
+							 const std::string& container_id)
+{
+	auto found = m_container_engine_by_type.find(type);
+	if(found == m_container_engine_by_type.end())
+	{
+		g_logger.format(sinsp_logger::SEV_ERROR,
+				"Container type %d not found when requesting size for %s",
+				type,
+				container_id.c_str());
+	}
+
+	g_logger.format(sinsp_logger::SEV_DEBUG,
+			"Request size for %s",
+			container_id.c_str());
+	found->second->update_with_size(container_id);
 }
 
 void sinsp_container_manager::cleanup()
@@ -507,9 +584,18 @@ void sinsp_container_manager::cleanup()
 	}
 }
 
+void sinsp_container_manager::set_docker_socket_path(std::string socket_path)
+{
+#if defined(HAS_CAPTURE)
+	libsinsp::container_engine::docker::set_docker_sock(std::move(socket_path));
+#endif
+}
+
 void sinsp_container_manager::set_query_docker_image_info(bool query_image_info)
 {
+#ifndef _WIN32
 	libsinsp::container_engine::docker_async_source::set_query_image_info(query_image_info);
+#endif
 }
 
 void sinsp_container_manager::set_cri_extra_queries(bool extra_queries)
@@ -532,3 +618,18 @@ void sinsp_container_manager::set_cri_timeout(int64_t timeout_ms)
 	libsinsp::container_engine::cri::set_cri_timeout(timeout_ms);
 #endif
 }
+
+void sinsp_container_manager::set_cri_async(bool async)
+{
+#if defined(HAS_CAPTURE)
+	libsinsp::container_engine::cri::set_async(async);
+#endif
+}
+
+void sinsp_container_manager::set_cri_delay(uint64_t delay_ms)
+{
+#if defined(HAS_CAPTURE)
+	libsinsp::container_engine::cri::set_cri_delay(delay_ms);
+#endif
+}
+
