@@ -17,6 +17,8 @@ limitations under the License.
 
 */
 
+#ifndef _WIN32
+
 #include "container_engine/docker.h"
 #include "cgroup_list_counter.h"
 #include "sinsp.h"
@@ -27,14 +29,20 @@ limitations under the License.
 
 using namespace libsinsp::container_engine;
 
-docker_async_source::docker_async_source(uint64_t max_wait_ms, uint64_t ttl_ms, sinsp *inspector)
+docker_async_source::docker_async_source(uint64_t max_wait_ms,
+					 uint64_t ttl_ms,
+					 container_cache_interface *cache
+#ifndef _WIN32
+	, std::string socket_path
+#endif
+	)
 	: async_key_value_source(max_wait_ms, ttl_ms),
-	  m_inspector(inspector),
-	  m_docker_unix_socket_path("/var/run/docker.sock"),
+	  m_cache(cache),
 #ifdef _WIN32
-	  m_api_version("/v1.30"),
+	  m_api_version("/v1.30")
 #else
 	  m_api_version("/v1.24"),
+	  m_docker_unix_socket_path(std::move(socket_path)),
 	  m_curlm(NULL),
 	  m_curl(NULL)
 #endif
@@ -53,21 +61,22 @@ docker_async_source::~docker_async_source()
 
 void docker_async_source::run_impl()
 {
-	std::string container_id;
+	docker_async_instruction instruction;
 
-	while (dequeue_next_key(container_id))
+	while (dequeue_next_key(instruction))
 	{
 		g_logger.format(sinsp_logger::SEV_DEBUG,
-				"docker_async (%s): Source dequeued key",
-				container_id.c_str());
+				"docker_async (%s : %s): Source dequeued key",
+				instruction.container_id.c_str(),
+				instruction.request_rw_size ? "true" : "false");
 
-		container_lookup_result res;
+		sinsp_container_info res;
 
-		res.m_successful = true;
-		res.m_container_info.m_type = CT_DOCKER;
-		res.m_container_info.m_id = container_id;
+		res.m_lookup_state = sinsp_container_lookup_state::SUCCESSFUL;
+		res.m_type = CT_DOCKER;
+		res.m_id = instruction.container_id;
 
-		if(!parse_docker(container_id, &res.m_container_info))
+		if(!parse_docker(instruction, res))
 		{
 			// This is not always an error e.g. when using
 			// containerd as the runtime. Since the cgroup
@@ -76,17 +85,17 @@ void docker_async_source::run_impl()
 			// fetch both.
 			g_logger.format(sinsp_logger::SEV_DEBUG,
 					"docker_async (%s): Failed to get Docker metadata, returning successful=false",
-					container_id.c_str());
-			res.m_successful = false;
+					instruction.container_id.c_str());
+			res.m_lookup_state = sinsp_container_lookup_state::FAILED;
 		}
 
 		g_logger.format(sinsp_logger::SEV_DEBUG,
 				"docker_async (%s): Parse successful, storing value",
-				container_id.c_str());
+				instruction.container_id.c_str());
 
 		// Return a result object either way, to ensure any
 		// new container callbacks are called.
-		store_value(container_id, res);
+		store_value(instruction, res);
 	}
 }
 
@@ -171,11 +180,11 @@ std::string docker_async_source::normalize_arg(const std::string &arg)
 }
 
 void docker_async_source::parse_healthcheck(const Json::Value &healthcheck_obj,
-					    sinsp_container_info *container)
+					    sinsp_container_info &container)
 {
 	g_logger.format(sinsp_logger::SEV_DEBUG,
 			"docker (%s): Trying to parse healthcheck from %s",
-			container->m_id.c_str(), Json::FastWriter().write(healthcheck_obj).c_str());
+			container.m_id.c_str(), Json::FastWriter().write(healthcheck_obj).c_str());
 
 	if(healthcheck_obj.isNull())
 	{
@@ -224,9 +233,9 @@ void docker_async_source::parse_healthcheck(const Json::Value &healthcheck_obj,
 
 		g_logger.format(sinsp_logger::SEV_DEBUG,
 				"docker (%s): Setting PT_HEALTHCHECK exe=%s nargs=%d",
-				container->m_id.c_str(), exe.c_str(), args.size());
+				container.m_id.c_str(), exe.c_str(), args.size());
 
-		container->m_health_probes.emplace_back(sinsp_container_info::container_health_probe::PT_HEALTHCHECK,
+		container.m_health_probes.emplace_back(sinsp_container_info::container_health_probe::PT_HEALTHCHECK,
 							std::move(exe),
 							std::move(args));
 	}
@@ -240,9 +249,9 @@ void docker_async_source::parse_healthcheck(const Json::Value &healthcheck_obj,
 
 		g_logger.format(sinsp_logger::SEV_DEBUG,
 				"docker (%s): Setting PT_HEALTHCHECK exe=%s nargs=%d",
-				container->m_id.c_str(), exe.c_str(), args.size());
+				container.m_id.c_str(), exe.c_str(), args.size());
 
-		container->m_health_probes.emplace_back(sinsp_container_info::container_health_probe::PT_HEALTHCHECK,
+		container.m_health_probes.emplace_back(sinsp_container_info::container_health_probe::PT_HEALTHCHECK,
 							std::move(exe),
 							std::move(args));
 	}
@@ -256,7 +265,7 @@ void docker_async_source::parse_healthcheck(const Json::Value &healthcheck_obj,
 
 bool docker_async_source::parse_liveness_readiness_probe(const Json::Value &probe_obj,
 							 sinsp_container_info::container_health_probe::probe_type ptype,
-							 sinsp_container_info *container)
+							 sinsp_container_info &container)
 {
 	if(probe_obj.isNull() ||
 	   !probe_obj.isMember("exec") ||
@@ -282,18 +291,63 @@ bool docker_async_source::parse_liveness_readiness_probe(const Json::Value &prob
 
 		g_logger.format(sinsp_logger::SEV_DEBUG,
 				"docker (%s): Setting %s exe=%s nargs=%d",
-				container->m_id.c_str(),
+				container.m_id.c_str(),
 				sinsp_container_info::container_health_probe::probe_type_names[ptype].c_str(),
 				exe.c_str(), args.size());
 
-		container->m_health_probes.emplace_back(ptype, std::move(exe), std::move(args));
+		container.m_health_probes.emplace_back(ptype, std::move(exe), std::move(args));
 	}
 
 	return true;
 }
 
+bool docker_async_source::get_sandbox_liveness_readiness_probes(const Json::Value &config_obj,
+								sinsp_container_info &container)
+{
+	std::string sandbox_container_id;
+	std::string sandbox_label = "io.kubernetes.sandbox.id";
+
+	if(config_obj.isNull() ||
+	   !config_obj.isMember("Labels") ||
+	   !config_obj["Labels"].isMember(sandbox_label))
+	{
+		SINSP_DEBUG("docker (%s): No sandbox label found, not copying liveness/readiness probes",
+			    container.m_id.c_str());
+		return false;
+	}
+
+	sandbox_container_id = config_obj["Labels"][sandbox_label].asString();
+
+	if(sandbox_container_id.size() > 12)
+	{
+		sandbox_container_id.resize(12);
+	}
+
+	sinsp_container_info::ptr_t sandbox_container = m_cache->get_container(sandbox_container_id);
+
+	if(!sandbox_container)
+	{
+		SINSP_DEBUG("docker (%s): Sandbox container %s doesn't exist, not copying liveness/readiness probes",
+			    container.m_id.c_str(), sandbox_container_id.c_str());
+		return false;
+	}
+
+	if(sandbox_container->m_health_probes.size() == 0)
+	{
+		SINSP_DEBUG("docker (%s): Sandbox container %s has no liveness/readiness probes, not copying",
+			    container.m_id.c_str(), sandbox_container_id.c_str());
+		return false;
+	}
+
+	SINSP_DEBUG("docker (%s): Copying liveness/readiness probes from sandbox container %s",
+		    container.m_id.c_str(), sandbox_container_id.c_str());
+	container.m_health_probes = sandbox_container->m_health_probes;
+
+	return true;
+}
+
 void docker_async_source::parse_health_probes(const Json::Value &config_obj,
-					      sinsp_container_info *container)
+					      sinsp_container_info &container)
 {
 	Json::Value spec;
 	bool liveness_readiness_added = false;
@@ -302,6 +356,10 @@ void docker_async_source::parse_health_probes(const Json::Value &config_obj,
 	// contains the probes.
 	if (get_k8s_pod_spec(config_obj, spec))
 	{
+		g_logger.format(sinsp_logger::SEV_DEBUG,
+				"docker (%s): Parsing liveness/readiness probes from pod spec",
+				container.m_id.c_str());
+
 		if(spec.isMember("livenessProbe"))
 		{
 			if(parse_liveness_readiness_probe(spec["livenessProbe"],
@@ -321,6 +379,17 @@ void docker_async_source::parse_health_probes(const Json::Value &config_obj,
 			}
 		}
 	}
+	// Otherwise, try to copy the liveness/readiness probe from the sandbox container, if it exists.
+	else if (get_sandbox_liveness_readiness_probes(config_obj, container))
+	{
+		liveness_readiness_added = true;
+	}
+	else
+	{
+		g_logger.format(sinsp_logger::SEV_DEBUG,
+				"docker (%s): No liveness/readiness probes found",
+				container.m_id.c_str());
+	}
 
 	// To avoid any confusion about containers that both refer to
 	// a healthcheck and liveness/readiness probe, we only
@@ -342,10 +411,10 @@ void docker_async_source::set_query_image_info(bool query_image_info)
 
 std::string docker::s_incomplete_info_name = "incomplete";
 
-bool docker::resolve(sinsp_container_manager* manager, sinsp_threadinfo* tinfo, bool query_os_for_missing_info)
+bool docker::resolve(sinsp_threadinfo *tinfo, bool query_os_for_missing_info)
 {
 	std::string container_id, container_name;
-	sinsp_container_info *existing_container_info;
+	container_cache_interface *cache = &container_cache();
 
 	if(!detect_docker(tinfo, container_id, container_name))
 	{
@@ -357,78 +426,69 @@ bool docker::resolve(sinsp_container_manager* manager, sinsp_threadinfo* tinfo,
 		g_logger.log("docker_async: Creating docker async source",
 			     sinsp_logger::SEV_DEBUG);
 		uint64_t max_wait_ms = 10000;
-		docker_async_source *src = new docker_async_source(docker_async_source::NO_WAIT_LOOKUP, max_wait_ms, manager->get_inspector());
+#ifdef _WIN32
+		docker_async_source *src = new docker_async_source(docker_async_source::NO_WAIT_LOOKUP, max_wait_ms, cache);
+#else
+		docker_async_source *src = new docker_async_source(docker_async_source::NO_WAIT_LOOKUP, max_wait_ms, cache, m_docker_sock);
+#endif
 		m_docker_info_source.reset(src);
 	}
 
 	tinfo->m_container_id = container_id;
 
-	existing_container_info = manager->get_container(container_id);
+	sinsp_container_info::ptr_t container_info = cache->get_container(container_id);
 
-	if(!existing_container_info)
+	if(!container_info)
 	{
-		// Add a minimal container_info object where only the
-		// container id, (possibly) name, and a container
-		// image = incomplete is filled in. This may be
-		// overidden later once parse_docker_async completes.
-		sinsp_container_info container_info;
-
-		g_logger.format(sinsp_logger::SEV_DEBUG,
-				"docker_async (%s): No existing container info, creating initial stub info",
-				container_id.c_str());
-
-		container_info.m_type = CT_DOCKER;
-		container_info.m_id = container_id;
-		container_info.m_name = container_name;
-		container_info.m_image = s_incomplete_info_name;
-		container_info.m_imageid = s_incomplete_info_name;
-		container_info.m_imagerepo = s_incomplete_info_name;
-		container_info.m_imagetag = s_incomplete_info_name;
-		container_info.m_imagedigest = s_incomplete_info_name;
-		container_info.m_metadata_complete = false;
-
-		manager->add_container(container_info, tinfo);
-
-		existing_container_info = manager->get_container(container_id);
-	}
+		if(!query_os_for_missing_info)
+		{
+			auto container = std::make_shared<sinsp_container_info>();
+			container->m_type = CT_DOCKER;
+			container->m_id = container_id;
+			cache->notify_new_container(*container);
+			return true;
+		}
 
 #ifdef HAS_CAPTURE
-	// Possibly start a lookup for this container info
-	if(!existing_container_info->m_metadata_complete &&
-	    query_os_for_missing_info)
-	{
-		// give docker a chance to return metadata for this container
-		parse_docker_async(manager->get_inspector(), container_id, manager);
-	}
+		if(cache->should_lookup(container_id, CT_DOCKER))
+		{
+			g_logger.format(sinsp_logger::SEV_DEBUG,
+					"docker_async (%s): No existing container info",
+					container_id.c_str());
+
+			// give docker a chance to return metadata for this container
+			cache->set_lookup_status(container_id, CT_DOCKER, sinsp_container_lookup_state::STARTED);
+			parse_docker_async(container_id, cache);
+		}
 #endif
+		return false;
+	}
 
 	// Returning true will prevent other container engines from
 	// trying to resolve the container, so only return true if we
 	// have complete metadata.
-	return existing_container_info->m_metadata_complete;
+	return container_info->is_successful();
 }
 
-void docker::parse_docker_async(sinsp *inspector, std::string &container_id, sinsp_container_manager *manager)
+void docker::parse_docker_async(const string& container_id, container_cache_interface *cache)
 {
-	auto cb = [manager](const std::string &container_id, const container_lookup_result &res)
-        {
+	auto cb = [cache](const docker_async_instruction& instruction, const sinsp_container_info& res)
+	{
 		g_logger.format(sinsp_logger::SEV_DEBUG,
-				"docker_async (%s): Source callback result successful=%s",
-				container_id.c_str(),
-				(res.m_successful ? "true" : "false"));
+				"docker_async (%s): Source callback result=%d",
+				instruction.container_id.c_str(),
+				res.m_lookup_state);
 
-		if(res.m_successful)
-		{
-			manager->notify_new_container(res.m_container_info);
-		}
+		cache->notify_new_container(res);
 	};
 
-        container_lookup_result result;
+        sinsp_container_info result;
 
-	if (m_docker_info_source->lookup(container_id, result, cb))
+	docker_async_instruction instruction(container_id, false /*don't request size*/);
+	if(m_docker_info_source->lookup(instruction, result, cb))
 	{
 		// if a previous lookup call already found the metadata, process it now
-		cb(container_id, result);
+		cb(instruction, result);
 
 		// This should *never* happen, as ttl is 0 (never wait)
 		g_logger.format(sinsp_logger::SEV_ERROR,
@@ -437,24 +497,31 @@ void docker::parse_docker_async(sinsp *inspector, std::string &container_id, sin
 	}
 }
 
-bool docker_async_source::parse_docker(std::string &container_id, sinsp_container_info *container)
+bool docker_async_source::parse_docker(const docker_async_instruction& instruction, sinsp_container_info& container)
 {
 	string json;
 
 	g_logger.format(sinsp_logger::SEV_DEBUG,
 			"docker_async (%s): Looking up info for container",
-			container_id.c_str());
+			instruction.container_id.c_str());
+
+	std::string request = build_request("/containers/" + instruction.container_id + "/json");
+	if(instruction.request_rw_size)
+	{
+		request += "?size=true";
+	}
+
+	docker_response resp = get_docker(request, json);
 
-	docker_response resp = get_docker(build_request("/containers/" + container_id + "/json"), json);
 	switch(resp) {
 		case docker_response::RESP_BAD_REQUEST:
 			g_logger.format(sinsp_logger::SEV_DEBUG,
 					"docker_async (%s): Initial url fetch failed, trying w/o api version",
-					container_id.c_str());
+					instruction.container_id.c_str());
 
 			m_api_version = "";
 			json = "";
-			resp = get_docker(build_request("/containers/" + container_id + "/json"), json);
+			resp = get_docker(build_request("/containers/" + instruction.container_id + "/json"), json);
 			if (resp == docker_response::RESP_OK)
 			{
 				break;
@@ -463,7 +530,7 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 		case docker_response::RESP_ERROR:
 			g_logger.format(sinsp_logger::SEV_DEBUG,
 					"docker_async (%s): Url fetch failed, returning false",
-					container_id.c_str());
+					instruction.container_id.c_str());
 
 			return false;
 
@@ -473,7 +540,7 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 
 	g_logger.format(sinsp_logger::SEV_DEBUG,
 			"docker_async (%s): Parsing containers response \"%s\"",
-			container_id.c_str(),
+			instruction.container_id.c_str(),
 			json.c_str());
 
 	Json::Value root;
@@ -483,7 +550,7 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 	{
 		g_logger.format(sinsp_logger::SEV_ERROR,
 				"docker_async (%s): Could not parse json \"%s\", returning false",
-				container_id.c_str(),
+				instruction.container_id.c_str(),
 				json.c_str());
 
 		ASSERT(false);
@@ -492,53 +559,59 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 
 	const Json::Value& config_obj = root["Config"];
 
-	container->m_image = config_obj["Image"].asString();
+	container.m_image = config_obj["Image"].asString();
 
 	string imgstr = root["Image"].asString();
 	size_t cpos = imgstr.find(":");
 	if(cpos != string::npos)
 	{
-		container->m_imageid = imgstr.substr(cpos + 1);
+		container.m_imageid = imgstr.substr(cpos + 1);
 	}
 
 	parse_health_probes(config_obj, container);
 
 	// containers can be spawned using just the imageID as image name,
 	// with or without the hash prefix (e.g. sha256:)
-	bool no_name = !container->m_imageid.empty() &&
-		strncmp(container->m_image.c_str(), container->m_imageid.c_str(),
-			MIN(container->m_image.length(), container->m_imageid.length())) == 0;
+	bool no_name = !container.m_imageid.empty() &&
+		strncmp(container.m_image.c_str(), container.m_imageid.c_str(),
+			MIN(container.m_image.length(), container.m_imageid.length())) == 0;
 	no_name |= !imgstr.empty() &&
-		strncmp(container->m_image.c_str(), imgstr.c_str(),
-			MIN(container->m_image.length(), imgstr.length())) == 0;
+		strncmp(container.m_image.c_str(), imgstr.c_str(),
+			MIN(container.m_image.length(), imgstr.length())) == 0;
 
 	if(!no_name || !m_query_image_info)
 	{
 		string hostname, port;
-		sinsp_utils::split_container_image(container->m_image,
+		sinsp_utils::split_container_image(container.m_image,
 						   hostname,
 						   port,
-						   container->m_imagerepo,
-						   container->m_imagetag,
-						   container->m_imagedigest,
+						   container.m_imagerepo,
+						   container.m_imagetag,
+						   container.m_imagedigest,
 						   false);
 	}
 
-	if(m_query_image_info && !container->m_imageid.empty() &&
-	   (no_name || container->m_imagedigest.empty() || (!container->m_imagedigest.empty() && container->m_imagetag.empty())))
+	if(m_query_image_info && !container.m_imageid.empty() &&
+	   (no_name || container.m_imagedigest.empty() || (!container.m_imagedigest.empty() && container.m_imagetag.empty())))
 	{
 		g_logger.format(sinsp_logger::SEV_DEBUG,
 				"docker_async (%s) image (%s): Fetching image info",
-				container_id.c_str(),
-				container->m_imageid.c_str());
+				instruction.container_id.c_str(),
+				container.m_imageid.c_str());
 
 		string img_json;
-		if(get_docker(build_request("/images/" + container->m_imageid + "/json?digests=1"), img_json) == docker_response::RESP_OK)
+		std::string url = "/images/" + container.m_imageid + "/json?digests=1";
+
+		g_logger.format(sinsp_logger::SEV_DEBUG,
+				"docker_async url: %s",
+				url.c_str());
+
+		if(get_docker(build_request(url), img_json) == docker_response::RESP_OK)
 		{
 			g_logger.format(sinsp_logger::SEV_DEBUG,
 					"docker_async (%s) image (%s): Image info fetch returned \"%s\"",
-					container_id.c_str(),
-					container->m_imageid.c_str(),
+					instruction.container_id.c_str(),
+					container.m_imageid.c_str(),
 					img_json.c_str());
 
 			Json::Value img_root;
@@ -546,8 +619,8 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 			{
 				// img_root["RepoDigests"] contains only digests for images pulled from registries.
 				// If an image gets retagged and is never pushed to any registry, we will not find
-				// that entry in container->m_imagerepo. Also, for locally built images we have the
-				// same issue. This leads to container->m_imagedigest being empty as well.
+				// that entry in container.m_imagerepo. Also, for locally built images we have the
+				// same issue. This leads to container.m_imagedigest being empty as well.
 				unordered_set<std::string> imageDigestSet;
 				for(const auto& rdig : img_root["RepoDigests"])
 				{
@@ -556,13 +629,13 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 						string repodigest = rdig.asString();
 						string digest = repodigest.substr(repodigest.find('@')+1);
 						imageDigestSet.insert(digest);
-						if(container->m_imagerepo.empty())
+						if(container.m_imagerepo.empty())
 						{
-							container->m_imagerepo = repodigest.substr(0, repodigest.find('@'));
+							container.m_imagerepo = repodigest.substr(0, repodigest.find('@'));
 						}
-						if(repodigest.find(container->m_imagerepo) != string::npos)
+						if(repodigest.find(container.m_imagerepo) != string::npos)
 						{
-							container->m_imagedigest = digest;
+							container.m_imagedigest = digest;
 							break;
 						}
 					}
@@ -572,13 +645,13 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 					if(rtag.isString())
 					{
 						string repotag = rtag.asString();
-						if(container->m_imagerepo.empty())
+						if(container.m_imagerepo.empty())
 						{
-							container->m_imagerepo = repotag.substr(0, repotag.rfind(":"));
+							container.m_imagerepo = repotag.substr(0, repotag.rfind(":"));
 						}
-						if(repotag.find(container->m_imagerepo) != string::npos)
+						if(repotag.find(container.m_imagerepo) != string::npos)
 						{
-							container->m_imagetag = repotag.substr(repotag.rfind(":")+1);
+							container.m_imagetag = repotag.substr(repotag.rfind(":")+1);
 							break;
 						}
 					}
@@ -586,16 +659,16 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 				// fix image digest for locally tagged images or multiple repo digests.
 				// Case 1: One repo digest with many tags.
 				// Case 2: Many repo digests with the same digest value.
-				if(container->m_imagedigest.empty() && imageDigestSet.size() == 1) {
-					container->m_imagedigest = *imageDigestSet.begin();
+				if(container.m_imagedigest.empty() && imageDigestSet.size() == 1) {
+					container.m_imagedigest = *imageDigestSet.begin();
 				}
 			}
 			else
 			{
 				g_logger.format(sinsp_logger::SEV_ERROR,
 						"docker_async (%s) image (%s): Could not parse json image info \"%s\"",
-						container_id.c_str(),
-						container->m_imageid.c_str(),
+						instruction.container_id.c_str(),
+						container.m_imageid.c_str(),
 						img_json.c_str());
 			}
 		}
@@ -603,25 +676,26 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 		{
 			g_logger.format(sinsp_logger::SEV_ERROR,
 					"docker_async (%s) image (%s): Could not fetch image info",
-					container_id.c_str(),
-					container->m_imageid.c_str());
+					instruction.container_id.c_str(),
+					container.m_imageid.c_str());
 		}
 
 	}
-	if(container->m_imagetag.empty())
+	if(container.m_imagetag.empty())
 	{
-		container->m_imagetag = "latest";
+		container.m_imagetag = "latest";
 	}
 
-	container->m_name = root["Name"].asString();
+	container.m_full_id = root["Id"].asString();
+	container.m_name = root["Name"].asString();
 	// k8s Docker container names could have '/' as the first character.
-	if(!container->m_name.empty() && container->m_name[0] == '/')
+	if(!container.m_name.empty() && container.m_name[0] == '/')
 	{
-		container->m_name = container->m_name.substr(1);
+		container.m_name = container.m_name.substr(1);
 	}
-	if(container->m_name.find("k8s_POD") == 0)
+	if(container.m_name.find("k8s_POD") == 0)
 	{
-		container->m_is_pod_sandbox = true;
+		container.m_is_pod_sandbox = true;
 	}
 
 	const Json::Value& net_obj = root["NetworkSettings"];
@@ -645,33 +719,35 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 			// separate thread so this is ok.
 			g_logger.format(sinsp_logger::SEV_DEBUG,
 					"docker_async (%s), secondary (%s): Doing blocking fetch of secondary container",
-					container_id.c_str(),
+					instruction.container_id.c_str(),
 					secondary_container_id.c_str());
 
-			if (parse_docker(secondary_container_id, &pcnt))
+			if(parse_docker(docker_async_instruction(secondary_container_id,
+								 false /*don't request size since we just need the IP*/),
+					pcnt))
 			{
 				g_logger.format(sinsp_logger::SEV_DEBUG,
 						"docker_async (%s), secondary (%s): Secondary fetch successful",
-						container_id.c_str(),
+						instruction.container_id.c_str(),
 						secondary_container_id.c_str());
-				container->m_container_ip = pcnt.m_container_ip;
+				container.m_container_ip = pcnt.m_container_ip;
 			}
 			else
 			{
 				g_logger.format(sinsp_logger::SEV_ERROR,
 						"docker_async (%s), secondary (%s): Secondary fetch failed",
-						container_id.c_str(),
+						instruction.container_id.c_str(),
 						secondary_container_id.c_str());
 			}
 		}
 	}
 	else
 	{
-		if(inet_pton(AF_INET, ip.c_str(), &container->m_container_ip) == -1)
+		if(inet_pton(AF_INET, ip.c_str(), &container.m_container_ip) == -1)
 		{
 			ASSERT(false);
 		}
-		container->m_container_ip = ntohl(container->m_container_ip);
+		container.m_container_ip = ntohl(container.m_container_ip);
 	}
 
 	vector<string> ports = net_obj["Ports"].getMemberNames();
@@ -704,7 +780,7 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 
 				port_mapping.m_container_port = container_port;
 				port_mapping.m_host_port = atoi(port.c_str());
-				container->m_port_mappings.push_back(port_mapping);
+				container.m_port_mappings.push_back(port_mapping);
 			}
 		}
 	}
@@ -713,7 +789,7 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 	for(vector<string>::const_iterator it = labels.begin(); it != labels.end(); ++it)
 	{
 		string val = config_obj["Labels"][*it].asString();
-		container->m_labels[*it] = val;
+		container.m_labels[*it] = val;
 	}
 
 	const Json::Value& env_vars = config_obj["Env"];
@@ -722,46 +798,70 @@ bool docker_async_source::parse_docker(std::string &container_id, sinsp_containe
 	{
 		if(env_var.isString())
 		{
-			container->m_env.emplace_back(env_var.asString());
+			container.m_env.emplace_back(env_var.asString());
 		}
 	}
 
 	const auto& host_config_obj = root["HostConfig"];
-	container->m_memory_limit = host_config_obj["Memory"].asInt64();
-	container->m_swap_limit = host_config_obj["MemorySwap"].asInt64();
+	container.m_memory_limit = host_config_obj["Memory"].asInt64();
+	container.m_swap_limit = host_config_obj["MemorySwap"].asInt64();
 	const auto cpu_shares = host_config_obj["CpuShares"].asInt64();
 	if(cpu_shares > 0)
 	{
-		container->m_cpu_shares = cpu_shares;
+		container.m_cpu_shares = cpu_shares;
 	}
-	container->m_cpu_quota = host_config_obj["CpuQuota"].asInt64();
+	container.m_cpu_quota = host_config_obj["CpuQuota"].asInt64();
 	const auto cpu_period = host_config_obj["CpuPeriod"].asInt64();
 	if(cpu_period > 0)
 	{
-		container->m_cpu_period = cpu_period;
+		container.m_cpu_period = cpu_period;
 	}
 	const auto cpuset_cpus = host_config_obj["CpusetCpus"].asString();
 	if (!cpuset_cpus.empty())
 	{
 		libsinsp::cgroup_list_counter counter;
-		container->m_cpuset_cpu_count = counter(cpuset_cpus.c_str(), sinsp_logger::SEV_DEBUG);
+		container.m_cpuset_cpu_count = counter(cpuset_cpus.c_str());
 	}
 	const Json::Value& privileged = host_config_obj["Privileged"];
 	if(!privileged.isNull() && privileged.isBool())
 	{
-		container->m_privileged = privileged.asBool();
+		container.m_privileged = privileged.asBool();
 	}
 
-	docker::parse_json_mounts(root["Mounts"], container->m_mounts);
+	docker::parse_json_mounts(root["Mounts"], container.m_mounts);
+
+	container.m_size_rw_bytes = root["SizeRw"].asInt64();
 
 #ifdef HAS_ANALYZER
-	sinsp_utils::find_env(container->m_sysdig_agent_conf, container->get_env(), "SYSDIG_AGENT_CONF");
-	// container->m_sysdig_agent_conf = get_docker_env(env_vars, "SYSDIG_AGENT_CONF");
+	sinsp_utils::find_env(container.m_sysdig_agent_conf, container.get_env(), "SYSDIG_AGENT_CONF");
+	// container.m_sysdig_agent_conf = get_docker_env(env_vars, "SYSDIG_AGENT_CONF");
 #endif
 
 	g_logger.format(sinsp_logger::SEV_DEBUG,
 			"docker_async (%s): parse_docker returning true",
-			container_id.c_str());
+			instruction.container_id.c_str());
 	return true;
 }
 
+void docker::update_with_size(const std::string &container_id)
+{
+	auto cb = [this](const docker_async_instruction& instruction, const sinsp_container_info& res) {
+		g_logger.format(sinsp_logger::SEV_DEBUG,
+				"docker_async (%s): with size callback result=%d",
+				instruction.container_id.c_str(),
+				res.m_lookup_state);
+
+		sinsp_container_info::ptr_t updated = make_shared<sinsp_container_info>(res);
+		container_cache().replace_container(updated);
+	};
+
+	g_logger.format(sinsp_logger::SEV_DEBUG,
+			"docker_async size request (%s)",
+			container_id.c_str());
+
+	sinsp_container_info result;
+	docker_async_instruction instruction(container_id, true /*request rw size*/);
+	(void)m_docker_info_source->lookup(instruction, result, cb);
+}
+
+#endif // _WIN32
