@@ -170,7 +170,7 @@ public:
 	/*!
 	  \brief Get the incremental number of this event.
 	*/
-	inline uint64_t get_num()
+	inline uint64_t get_num() const
 	{
 		return m_evtnum;
 	}
@@ -339,18 +339,49 @@ public:
 #endif
 
 	/*!
-	  \brief Return whether or not falco should consider this
-	  event. (Generally, these events are automatically filtered
-	  out, but some events related to internal tracking are returned by next() anyway).
+	  \brief Return whether or not a simple consumer that privileges low overhead to
+	  full event capture should consider this event. (Generally, these events are
+	  automatically filtered out, but some events related to internal tracking are
+	  returned by next() anyway).
 	*/
 
-	bool falco_consider();
+	bool simple_consumer_consider();
 
 	inline uint16_t get_source()
 	{
 		return ESRC_SINSP;
 	}
 
+	/*!
+	  \brief Returns true if this event represents a system call error,
+	         false otherwise.
+	*/
+	bool is_syscall_error() const;
+
+	/*!
+	  \brief Returns true if this event represents a file open system
+	         call error, false otherwise.
+
+          Precondition: is_syscall_error() must return true.
+	*/
+	bool is_file_open_error() const;
+
+	/*!
+	  \brief Returns true if this event represents a file-related system
+	         call error (including open errors), false otherwise.
+
+	  Precondition: is_syscall_error() must return true.
+	*/
+	bool is_file_error() const;
+
+	/*!
+	  \brief Returns true if this event represents a network-related system
+	         call error, false otherwise.
+
+	  Precondition: is_syscall_error() must return true.
+	*/
+	bool is_network_error() const;
+
 // Doxygen doesn't understand VISIBILITY_PRIVATE
 #ifdef _DOXYGEN
 private:
@@ -363,17 +394,21 @@ private:
 
 	const char* get_param_value_str(const char* name, OUT const char** resolved_str, param_fmt fmt = PF_NORMAL);
 
-	inline void init()
+	inline void init_keep_threadinfo()
 	{
 		m_flags = EF_NONE;
 		m_info = &(m_event_info_table[m_pevt->type]);
-		m_tinfo_ref.reset();
-		m_tinfo = NULL;
 		m_fdinfo = NULL;
 		m_fdinfo_name_changed = false;
 		m_iosize = 0;
 		m_poriginal_evt = NULL;
 	}
+	inline void init()
+	{
+		init_keep_threadinfo();
+		m_tinfo_ref.reset();
+		m_tinfo = NULL;
+	}
 	inline void init(uint8_t* evdata, uint16_t cpuid)
 	{
 		m_flags = EF_NONE;
