@@ -1257,7 +1257,7 @@ const filtercheck_field_info* sinsp_filter_check::get_field_info()
 
 bool sinsp_filter_check::flt_compare(cmpop op, ppm_param_type type, void* operand1, uint32_t op1_len, uint32_t op2_len)
 {
-	if (op == CO_IN || op == CO_PMATCH)
+	if (op == CO_IN || op == CO_PMATCH || op == CO_INTERSECTS)
 	{
 		// Certain filterchecks can't be done as a set
 		// membership test/group match. For these, just loop over the
@@ -1295,8 +1295,13 @@ bool sinsp_filter_check::flt_compare(cmpop op, ppm_param_type type, void* operan
 
 			filter_value_t item((uint8_t *) operand1, op1_len);
 
-			if (op == CO_IN)
+			if (op == CO_IN || op == CO_INTERSECTS)
 			{
+				// CO_INTERSECTS is really more interesting when a filtercheck can extract
+				// multiple values, and you're comparing the set of extracted values
+				// against the set of rhs values. sinsp_filter_checks only extract a
+				// single value, so CO_INTERSECTS is really the same as CO_IN.
+
 				if(op1_len >= m_val_storages_min_size &&
 				   op1_len <= m_val_storages_max_size &&
 				   m_val_storages_members.find(item) != m_val_storages_members.end())
@@ -1689,6 +1694,11 @@ cmpop sinsp_filter_compiler::next_comparison_operator()
 		m_scanpos += 2;
 		return CO_IN;
 	}
+	else if(compare_no_consume("intersects"))
+	{
+		m_scanpos += 10;
+		return CO_INTERSECTS;
+	}
 	else if(compare_no_consume("pmatch"))
 	{
 		m_scanpos += 6;
@@ -1743,7 +1753,7 @@ void sinsp_filter_compiler::parse_check()
 
 	chk->parse_field_name((char *)&operand1[0], true, true);
 
-	if(co == CO_IN || co == CO_PMATCH)
+	if(co == CO_IN || co == CO_INTERSECTS || co == CO_PMATCH)
 	{
 		//
 		// Skip spaces
@@ -1755,7 +1765,7 @@ void sinsp_filter_compiler::parse_check()
 
 		if(m_fltstr[m_scanpos] != '(')
 		{
-			throw sinsp_exception("expected '(' after 'in/pmatch' operand");
+			throw sinsp_exception("expected '(' after 'in/intersects/pmatch' operand");
 		}
 
 		//
@@ -1900,10 +1910,10 @@ sinsp_filter* sinsp_filter_compiler::compile()
 	{
 		return compile_();
 	}
-	catch(sinsp_exception& e)
+	catch(const sinsp_exception& e)
 	{
 		delete m_filter;
-		throw e;
+		throw;
 	}
 	catch(...)
 	{
