@@ -28,6 +28,7 @@ limitations under the License.
 #include <sys/time.h>
 #endif // _WIN32
 
+#include "scap_open_exception.h"
 #include "sinsp.h"
 #include "sinsp_int.h"
 #include "sinsp_auth.h"
@@ -42,6 +43,7 @@ limitations under the License.
 #include "k8s_api_handler.h"
 #ifdef HAS_CAPTURE
 #include <curl/curl.h>
+#include <mntent.h>
 #endif
 #endif
 
@@ -62,6 +64,7 @@ void on_new_entry_from_proc(void* context, scap_t* handle, int64_t tid, scap_thr
 // sinsp implementation
 ///////////////////////////////////////////////////////////////////////////////
 sinsp::sinsp() :
+	m_external_event_processor(),
 	m_evt(this),
 	m_lastevent_ts(0),
 	m_container_manager(this),
@@ -87,9 +90,6 @@ sinsp::sinsp() :
 	m_inactive_container_scan_time_ns = DEFAULT_INACTIVE_CONTAINER_SCAN_TIME_S * ONE_SECOND_IN_NS;
 	m_cycle_writer = NULL;
 	m_write_cycling = false;
-#ifdef HAS_ANALYZER
-	m_analyzer = NULL;
-#endif
 
 #ifdef HAS_FILTERING
 	m_filter = NULL;
@@ -103,6 +103,7 @@ sinsp::sinsp() :
 #endif
 	m_n_proc_lookups = 0;
 	m_n_proc_lookups_duration_ns = 0;
+	m_n_main_thread_lookups = 0;
 	m_snaplen = DEFAULT_SNAPLEN;
 	m_buffer_format = sinsp_evt::PF_NORMAL;
 	m_input_fd = 0;
@@ -311,6 +312,7 @@ void sinsp::init()
 	m_fds_to_remove->clear();
 	m_n_proc_lookups = 0;
 	m_n_proc_lookups_duration_ns = 0;
+	m_n_main_thread_lookups = 0;
 
 	//
 	// Return the tracers to the pool and clear the tracers list
@@ -358,16 +360,10 @@ void sinsp::init()
 			}
 		}
 
-
-#ifdef HAS_ANALYZER
-		//
-		// Notify the analyzer that we're starting
-		//
-		if(m_analyzer)
+		if (m_external_event_processor)
 		{
-			m_analyzer->on_capture_start();
+			m_external_event_processor->on_capture_start();
 		}
-#endif
 
 		//
 		// Rewind, reset the event count, and consume the exact number of events
@@ -400,16 +396,10 @@ void sinsp::init()
 	//
 	m_thread_manager->fix_sockets_coming_from_proc();
 
-#ifdef HAS_ANALYZER
-	//
-	// Notify the analyzer that we're starting
-	//
-	if(m_analyzer)
+	if (m_external_event_processor)
 	{
-		m_analyzer->on_capture_start();
+		m_external_event_processor->on_capture_start();
 	}
-#endif
-
 	//
 	// If m_snaplen was modified, we set snaplen now
 	//
@@ -497,7 +487,7 @@ void sinsp::open(uint32_t timeout_ms)
 
 	if(m_h == NULL)
 	{
-		throw sinsp_exception(error, scap_rc);
+		throw scap_open_exception(error, scap_rc);
 	}
 
 	scap_set_refresh_proc_table_when_saving(m_h, !m_filter_proc_table_when_saving);
@@ -537,7 +527,7 @@ void sinsp::open_nodriver()
 
 	if(m_h == NULL)
 	{
-		throw sinsp_exception(error, scap_rc);
+		throw scap_open_exception(error, scap_rc);
 	}
 
 	scap_set_refresh_proc_table_when_saving(m_h, !m_filter_proc_table_when_saving);
@@ -676,7 +666,7 @@ void sinsp::open_int()
 
 	if(m_h == NULL)
 	{
-		throw sinsp_exception(error, scap_rc);
+		throw scap_open_exception(error, scap_rc);
 	}
 
 	if(m_input_fd != 0)
@@ -838,7 +828,7 @@ void sinsp::on_new_entry_from_proc(void* context,
 	if(fdinfo == NULL)
 	{
 		bool thread_added = false;
-		sinsp_threadinfo* newti = new sinsp_threadinfo(this);
+		sinsp_threadinfo* newti = build_threadinfo();
 		newti->init(tinfo);
 		if(is_nodriver())
 		{
@@ -862,7 +852,7 @@ void sinsp::on_new_entry_from_proc(void* context,
 
 		if(!sinsp_tinfo)
 		{
-			sinsp_threadinfo* newti = new sinsp_threadinfo(this);
+			sinsp_threadinfo* newti = build_threadinfo();
 			newti->init(tinfo);
 
 			if (!m_thread_manager->add_thread(newti, true)) {
@@ -905,7 +895,7 @@ void sinsp::import_thread_table()
 	//
 	HASH_ITER(hh, table, pi, tpi)
 	{
-		sinsp_threadinfo* newti = new sinsp_threadinfo(this);
+		sinsp_threadinfo* newti = build_threadinfo();
 		newti->init(pi);
 		m_thread_manager->add_thread(newti, true);
 	}
@@ -1006,10 +996,6 @@ void schedule_next_threadinfo_evt(sinsp* _this, void* data)
 
 			_this->add_meta_event(&mei->m_pievt);
 		}
-		else if(mei->m_cur_procinfo_evt == (int32_t)mei->m_n_procinfo_evts)
-		{
-			_this->add_meta_event(mei->m_next_evt);
-		}
 
 		break;
 	}
@@ -1041,6 +1027,18 @@ void sinsp::restart_capture_at_filepos(uint64_t filepos)
 	}
 }
 
+uint64_t sinsp::max_buf_used()
+{
+	if(m_h)
+	{
+		return scap_max_buf_used(m_h);
+	}
+	else
+	{
+		return 0;
+	}
+}
+
 int32_t sinsp::next(OUT sinsp_evt **puevt)
 {
 	sinsp_evt* evt;
@@ -1060,11 +1058,13 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 			m_meta_event_callback(this, m_meta_event_callback_data);
 		}
 	}
+#ifndef _WIN32
 	else if (m_pending_container_evts.try_pop(m_container_evt))
 	{
 		res = SCAP_SUCCESS;
 		evt = m_container_evt.get();
 	}
+#endif
 	else
 	{
 		evt = &m_evt;
@@ -1092,23 +1092,19 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 		{
 			if(res == SCAP_TIMEOUT)
 			{
-	#ifdef HAS_ANALYZER
-				if(m_analyzer)
+				if (m_external_event_processor)
 				{
-					m_analyzer->process_event(NULL, analyzer_emitter::DF_TIMEOUT);
+					m_external_event_processor->process_event(NULL, libsinsp::EVENT_RETURN_TIMEOUT);
 				}
-	#endif
 				*puevt = NULL;
 				return res;
 			}
 			else if(res == SCAP_EOF)
 			{
-	#ifdef HAS_ANALYZER
-				if(m_analyzer)
+				if (m_external_event_processor)
 				{
-					m_analyzer->process_event(NULL, analyzer_emitter::DF_EOF);
+					m_external_event_processor->process_event(NULL, libsinsp::EVENT_RETURN_EOF);
 				}
-	#endif
 			}
 			else if(res == SCAP_UNEXPECTED_BLOCK)
 			{
@@ -1165,7 +1161,6 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 						m_meinfo.m_cur_procinfo_evt = -1;
 
 						m_meinfo.m_piscapevt->ts = m_next_flush_time_ns - (ONE_SECOND_IN_NS + 1);
-						m_meinfo.m_next_evt = &m_evt;
 						add_meta_event_callback(&schedule_next_threadinfo_evt, &m_meinfo);
 						schedule_next_threadinfo_evt(this, &m_meinfo);
 					}
@@ -1360,30 +1355,10 @@ int32_t sinsp::next(OUT sinsp_evt **puevt)
 	//
 	// Run the analysis engine
 	//
-#ifdef HAS_ANALYZER
-	if(m_analyzer)
+	if (m_external_event_processor)
 	{
-#ifdef SIMULATE_DROP_MODE
-		if(!sd || m_isdropping || sw)
-		{
-			if(m_isdropping)
-			{
-				m_analyzer->process_event(evt, analyzer_emitter::DF_FORCE_FLUSH);
-			}
-			else if(sw)
-			{
-				m_analyzer->process_event(evt, analyzer_emitter::DF_FORCE_FLUSH_BUT_DONT_EMIT);
-			}
-			else
-			{
-				m_analyzer->process_event(evt, analyzer_emitter::DF_FORCE_NOFLUSH);
-			}
-		}
-#else // SIMULATE_DROP_MODE
-		m_analyzer->process_event(evt, analyzer_emitter::DF_NONE);
-#endif // SIMULATE_DROP_MODE
+		m_external_event_processor->process_event(evt, libsinsp::EVENT_RETURN_NONE);
 	}
-#endif
 
 	// Clean parse related event data after analyzer did its parsing too
 	m_parser->event_cleanup(evt);
@@ -1430,7 +1405,7 @@ sinsp_threadinfo* sinsp::get_thread(int64_t tid, bool query_os_if_not_found, boo
 	return &*get_thread_ref(tid, query_os_if_not_found, lookup_only);
 }
 
-threadinfo_map_t::ptr_t sinsp::get_thread_ref(int64_t tid, bool query_os_if_not_found, bool lookup_only)
+threadinfo_map_t::ptr_t sinsp::get_thread_ref(int64_t tid, bool query_os_if_not_found, bool lookup_only, bool main_thread)
 {
 	auto sinsp_proc = find_thread(tid, lookup_only);
 
@@ -1452,10 +1427,15 @@ threadinfo_map_t::ptr_t sinsp::get_thread_ref(int64_t tid, bool query_os_if_not_
 		}
 
 		scap_threadinfo* scap_proc = NULL;
-		sinsp_threadinfo* newti = new sinsp_threadinfo(this);
+		sinsp_threadinfo* newti = build_threadinfo();
 
 		m_n_proc_lookups++;
 
+		if(main_thread)
+		{
+			m_n_main_thread_lookups++;
+		}
+
 		if(m_n_proc_lookups == m_max_n_proc_lookups)
 		{
 			g_logger.format(sinsp_logger::SEV_INFO, "Reached max process lookup number, duration=%" PRIu64 "ms",
@@ -1589,6 +1569,11 @@ void sinsp::add_suppressed_comms(scap_open_args &oargs)
 	oargs.suppressed_comms[i++] = NULL;
 }
 
+void sinsp::set_docker_socket_path(std::string socket_path)
+{
+	m_container_manager.set_docker_socket_path(std::move(socket_path));
+}
+
 void sinsp::set_query_docker_image_info(bool query_image_info)
 {
 	m_container_manager.set_query_docker_image_info(query_image_info);
@@ -1609,6 +1594,16 @@ void sinsp::set_cri_timeout(int64_t timeout_ms)
 	m_container_manager.set_cri_timeout(timeout_ms);
 }
 
+void sinsp::set_cri_async(bool async)
+{
+	m_container_manager.set_cri_async(async);
+}
+
+void sinsp::set_cri_delay(uint64_t delay_ms)
+{
+	m_container_manager.set_cri_delay(delay_ms);
+}
+
 void sinsp::set_snaplen(uint32_t snaplen)
 {
 	//
@@ -1834,7 +1829,7 @@ uint32_t sinsp::reserve_thread_memory(uint32_t size)
 	return m_thread_privatestate_manager.reserve(size);
 }
 
-void sinsp::get_capture_stats(scap_stats* stats)
+void sinsp::get_capture_stats(scap_stats* stats) const
 {
 	if(scap_get_stats(m_h, stats) != SCAP_SUCCESS)
 	{
@@ -2295,7 +2290,7 @@ void sinsp::k8s_discover_ext()
 			}
 		}
 	}
-	catch(std::exception& ex)
+	catch(const std::exception& ex)
 	{
 		g_logger.log(std::string("K8s API extensions handler error: ").append(ex.what()),
 					 sinsp_logger::SEV_ERROR);
@@ -2368,7 +2363,7 @@ void sinsp::update_k8s_state()
 			}
 		}
 	}
-	catch(std::exception& e)
+	catch(const std::exception& e)
 	{
 		g_logger.log(std::string("Error fetching K8s data: ").append(e.what()), sinsp_logger::SEV_ERROR);
 		throw;
@@ -2399,7 +2394,7 @@ bool sinsp::get_mesos_data()
 			last_mesos_refresh = now;
 		}
 	}
-	catch(std::exception& ex)
+	catch(const std::exception& ex)
 	{
 		g_logger.log(std::string("Mesos exception: ") + ex.what(), sinsp_logger::SEV_ERROR);
 		delete m_mesos_client;
@@ -2532,3 +2527,62 @@ bool sinsp_thread_manager::remove_inactive_threads()
 
 	return res;
 }
+
+#ifdef HAS_CAPTURE
+std::shared_ptr<std::string> sinsp::lookup_cgroup_dir(const string& subsys)
+{
+	shared_ptr<string> cgroup_dir;
+	static std::unordered_map<std::string, std::shared_ptr<std::string>> cgroup_dir_cache;
+
+	const auto& it = cgroup_dir_cache.find(subsys);
+	if(it != cgroup_dir_cache.end())
+	{
+		return it->second;
+	}
+
+	// Look for mount point of cgroup filesystem
+	// It should be already mounted on the host or by
+	// our docker-entrypoint.sh script
+	if(strcmp(scap_get_host_root(), "") != 0)
+	{
+		// We are inside our container, so we should use the directory
+		// mounted by it
+		auto cgroup = std::string(scap_get_host_root()) + "/cgroup/" + subsys;
+		cgroup_dir = std::make_shared<std::string>(cgroup);
+	}
+	else
+	{
+		struct mntent mntent_buf = {};
+		char mntent_string_buf[4096];
+		FILE* fp = setmntent("/proc/mounts", "r");
+		struct mntent* entry = getmntent_r(fp, &mntent_buf,
+			mntent_string_buf, sizeof(mntent_string_buf));
+		while(entry != nullptr)
+		{
+			if(strcmp(entry->mnt_type, "cgroup") == 0 &&
+			   hasmntopt(entry, subsys.c_str()) != NULL)
+			{
+				cgroup_dir = std::make_shared<std::string>(entry->mnt_dir);
+				break;
+			}
+			entry = getmntent(fp);
+		}
+		endmntent(fp);
+	}
+	if(!cgroup_dir)
+	{
+		return std::make_shared<std::string>();
+	}
+	else
+	{
+		cgroup_dir_cache[subsys] = cgroup_dir;
+		return cgroup_dir;
+	}
+}
+#endif
+
+sinsp_threadinfo*
+libsinsp::event_processor::build_threadinfo(sinsp* inspector)
+{
+	return new sinsp_threadinfo(inspector);
+}
