@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2013-2018 Draios Inc dba Sysdig.
+Copyright (C) 2013-2019 Sysdig Inc.
 
 This file is part of sysdig.
 
@@ -44,15 +44,17 @@ limitations under the License.
 #pragma once
 
 #include "capture_stats_source.h"
+#include "container_engine/wmi_handle_source.h"
 
 #ifdef _WIN32
 #pragma warning(disable: 4251 4200 4221 4190)
-#endif
-
+#else
 #include "tbb/concurrent_queue.h"
+#endif
 
 #include "sinsp_inet.h"
 #include "sinsp_public.h"
+#include "sinsp_exception.h"
 
 #define __STDC_FORMAT_MACROS
 
@@ -96,6 +98,7 @@ using namespace std;
 #include "eventformatter.h"
 #include "sinsp_pd_callback_type.h"
 
+#include "include/sinsp_external_processor.h"
 class sinsp_partial_transaction;
 class sinsp_parser;
 class sinsp_analyzer;
@@ -153,51 +156,6 @@ public:
 	uint32_t m_flags;
 };
 
-/*!
-  \brief sinsp library exception.
-*/
-struct sinsp_exception : std::exception
-{
-	sinsp_exception()
-	{
-	}
-
-	~sinsp_exception() throw()
-	{
-	}
-
-	sinsp_exception(string error_str)
-	{
-		m_error_str = error_str;
-	}
-
-	sinsp_exception(string error_str, int32_t scap_rc)
-	{
-		m_error_str = error_str;
-		m_scap_rc = scap_rc;
-	}
-
-	char const* what() const throw()
-	{
-		return m_error_str.c_str();
-	}
-
-	int32_t scap_rc()
-	{
-		return m_scap_rc;
-	}
-
-	string m_error_str;
-	int32_t m_scap_rc;
-};
-
-/*!
-  \brief sinsp library exception.
-*/
-struct sinsp_capture_interrupt_exception : sinsp_exception
-{
-};
-
 /*!
   \brief The default way an event is converted to string by the library
 */
@@ -216,7 +174,6 @@ public:
 	uint64_t m_n_procinfo_evts;
 	int64_t m_cur_procinfo_evt;
 	ppm_proclist_info* m_pli;
-	sinsp_evt* m_next_evt;
 };
 
 /** @defgroup inspector Main library
@@ -231,7 +188,7 @@ public:
   - event retrieval
   - setting capture filters
 */
-class SINSP_PUBLIC sinsp : public capture_stats_source
+class SINSP_PUBLIC sinsp : public capture_stats_source, public wmi_handle_source
 {
 public:
 	typedef std::shared_ptr<sinsp> ptr;
@@ -296,6 +253,11 @@ public:
 	*/
 	virtual int32_t next(OUT sinsp_evt **evt);
 
+	/*!
+	  \brief Get the maximum number of bytes currently in use by any CPU buffer
+     */
+	uint64_t max_buf_used();
+
 	/*!
 	  \brief Get the number of events that have been captured and processed
 	   since the call to \ref open()
@@ -503,7 +465,7 @@ public:
 	   of failure.
 	*/
 	sinsp_threadinfo* get_thread(int64_t tid, bool query_os_if_not_found, bool lookup_only);
-	threadinfo_map_t::ptr_t get_thread_ref(int64_t tid, bool query_os_if_not_found, bool lookup_only);
+	threadinfo_map_t::ptr_t get_thread_ref(int64_t tid, bool query_os_if_not_found, bool lookup_only, bool main_thread=false);
 
 	/*!
 	  \brief Return the table with all the machine users.
@@ -547,7 +509,7 @@ public:
 
 	  \note this call won't work on file captures.
 	*/
-	void get_capture_stats(scap_stats* stats) override;
+	void get_capture_stats(scap_stats* stats) const override;
 
 	void set_max_thread_table_size(uint32_t value);
 
@@ -555,9 +517,28 @@ public:
 	sinsp_stats get_stats();
 #endif
 
-#ifdef HAS_ANALYZER
-	sinsp_analyzer* m_analyzer;
-#endif
+	libsinsp::event_processor* m_external_event_processor;
+
+	sinsp_threadinfo* build_threadinfo()
+    {
+        return m_external_event_processor ? m_external_event_processor->build_threadinfo(this)
+                                          : new sinsp_threadinfo(this);
+    } 
+
+	/*!
+	  \brief registers external event processor.
+	  After this, callbacks on libsinsp::event_processor will happen at
+	  the appropriate times. This registration must happen before calling open.
+	*/
+	void register_external_event_processor(libsinsp::event_processor& processor)
+	{
+		m_external_event_processor = &processor;
+	}
+
+	libsinsp::event_processor* get_external_event_processor() const
+	{
+		return m_external_event_processor;
+	}
 
 	/*!
 	  \brief Return the event and system call information tables.
@@ -867,25 +848,29 @@ public:
 	bool is_bpf_enabled();
 
 	static unsigned num_possible_cpus();
+
+#ifdef HAS_CAPTURE
+	static std::shared_ptr<std::string> lookup_cgroup_dir(const std::string& subsys);
+#endif
 #ifdef CYGWING_AGENT
-	wh_t* get_wmi_handle()
+	wh_t* get_wmi_handle() override
 	{
 		return scap_get_wmi_handle(m_h);
 	}
 #endif
 
-	static inline bool falco_consider_evtnum(uint16_t etype)
+	static inline bool simple_consumer_consider_evtnum(uint16_t etype)
 	{
 		enum ppm_event_flags flags = g_infotables.m_event_info[etype].flags;
 
-		return ! (flags & sinsp::falco_skip_flags());
+		return ! (flags & sinsp::simple_consumer_skip_flags());
 	}
 
-	static inline bool falco_consider_syscallid(uint16_t scid)
+	static inline bool simple_consumer_consider_syscallid(uint16_t scid)
 	{
 		enum ppm_event_flags flags = g_infotables.m_syscall_info_table[scid].flags;
 
-		return ! (flags & sinsp::falco_skip_flags());
+		return ! (flags & sinsp::simple_consumer_skip_flags());
 	}
 
 	// Add comm to the list of comms for which the inspector
@@ -894,6 +879,7 @@ public:
 
 	bool check_suppressed(int64_t tid);
 
+	void set_docker_socket_path(std::string socket_path);
 	void set_query_docker_image_info(bool query_image_info);
 
 	void set_cri_extra_queries(bool extra_queries);
@@ -904,6 +890,8 @@ public:
 
 	void set_cri_socket_path(const std::string& path);
 	void set_cri_timeout(int64_t timeout_ms);
+	void set_cri_async(bool async);
+	void set_cri_delay(uint64_t delay_ms);
 
 VISIBILITY_PROTECTED
 	bool add_thread(const sinsp_threadinfo *ptinfo);
@@ -914,9 +902,9 @@ VISIBILITY_PROTECTED
 
 VISIBILITY_PRIVATE
 
-        static inline ppm_event_flags falco_skip_flags()
+        static inline ppm_event_flags simple_consumer_skip_flags()
         {
-		return (ppm_event_flags) (EF_SKIPPARSERESET | EF_UNUSED | EF_DROP_FALCO);
+		return (ppm_event_flags) (EF_SKIPPARSERESET | EF_UNUSED | EF_DROP_SIMPLE_CONS);
         }
 // Doxygen doesn't understand VISIBILITY_PRIVATE
 #ifdef _DOXYGEN
@@ -1116,6 +1104,7 @@ public:
 #endif
 	int32_t m_n_proc_lookups;
 	uint64_t m_n_proc_lookups_duration_ns;
+	int32_t m_n_main_thread_lookups;
 	int32_t m_max_n_proc_lookups = -1;
 	int32_t m_max_n_proc_socket_lookups = -1;
 #ifdef HAS_ANALYZER
@@ -1198,7 +1187,9 @@ public:
 	// A queue of pending container events. Written from async
 	// callbacks that occur after looking up container
 	// information, read from sinsp::next().
+#ifndef _WIN32
 	tbb::concurrent_queue<shared_ptr<sinsp_evt>> m_pending_container_evts;
+#endif
 
 	// Holds an event dequeued from the above queue
 	std::shared_ptr<sinsp_evt> m_container_evt;
