@@ -40,19 +40,11 @@ static void copy_ipv6_address(uint32_t* dest, uint32_t* src)
 ///////////////////////////////////////////////////////////////////////////////
 // sinsp_threadinfo implementation
 ///////////////////////////////////////////////////////////////////////////////
-sinsp_threadinfo::sinsp_threadinfo() :
-	m_fdtable(NULL)
-{
-	m_inspector = NULL;
-	m_tracer_parser = NULL;
-	init();
-}
-
-sinsp_threadinfo::sinsp_threadinfo(sinsp *inspector) :
+sinsp_threadinfo::sinsp_threadinfo(sinsp* inspector) :
+	m_tracer_parser(NULL),
+	m_inspector(inspector),
 	m_fdtable(inspector)
 {
-	m_inspector = inspector;
-	m_tracer_parser = NULL;
 	init();
 }
 
@@ -84,9 +76,8 @@ void sinsp_threadinfo::init()
 	m_last_latency_entertime = 0;
 	m_latency = 0;
 #endif
-	m_ainfo = NULL;
 	m_program_hash = 0;
-	m_program_hash_falco = 0;
+	m_program_hash_scripts = 0;
 	m_lastevent_data = NULL;
 	m_parent_loop_detected = false;
 	m_tty = 0;
@@ -171,9 +162,9 @@ void sinsp_threadinfo::compute_program_hash()
 	auto rem_len = MAX_PROG_HASH_LEN - (m_exe.size() + m_container_id.size());
 
 	//
-	// By default, the falco hash is just exe+container
+	// By default, the scripts hash is just exe+container
 	//
-	m_program_hash_falco = curr_hash;
+	m_program_hash_scripts = curr_hash;
 
 	//
 	// The program hash includes the arguments as well
@@ -194,7 +185,7 @@ void sinsp_threadinfo::compute_program_hash()
 
 	//
 	// For some specific processes (essentially the scripting languages)
-	// we include the arguments in the falco hash as well
+	// we include the arguments in the scripts hash as well
 	//
 	if(m_comm.size() == 4)
 	{
@@ -203,14 +194,14 @@ void sinsp_threadinfo::compute_program_hash()
 		if(ncomm == STR_AS_NUM_JAVA || ncomm == STR_AS_NUM_RUBY ||
 			ncomm == STR_AS_NUM_PERL || ncomm == STR_AS_NUM_NODE)
 		{
-			m_program_hash_falco = m_program_hash;
+			m_program_hash_scripts = m_program_hash;
 		}
 	}
 	else if(m_comm.size() >= 6)
 	{
 		if(m_comm.substr(0, 6) == "python")
 		{
-			m_program_hash_falco = m_program_hash;
+			m_program_hash_scripts = m_program_hash;
 		}
 	}
 }
@@ -883,7 +874,7 @@ double sinsp_threadinfo::get_fd_usage_pct_d()
 	}
 }
 
-uint64_t sinsp_threadinfo::get_fd_opencount()
+uint64_t sinsp_threadinfo::get_fd_opencount() const
 {
 	return get_main_thread()->m_fdtable.size();
 }
@@ -893,6 +884,21 @@ uint64_t sinsp_threadinfo::get_fd_limit()
 	return get_main_thread()->m_fdlimit;
 }
 
+const std::string& sinsp_threadinfo::get_cgroup(const std::string& subsys) const
+{
+	static const std::string notfound = "/";
+
+	for(const auto& it : m_cgroups)
+	{
+		if(it.first == subsys)
+		{
+			return it.second;
+		}
+	}
+
+	return notfound;
+}
+
 void sinsp_threadinfo::traverse_parent_state(visitor_func_t &visitor)
 {
 	// Use two pointers starting at this, traversing the parent
@@ -962,54 +968,11 @@ bool sinsp_threadinfo::is_health_probe()
 		m_category == sinsp_threadinfo::CAT_READINESS_PROBE);
 }
 
-shared_ptr<sinsp_threadinfo> sinsp_threadinfo::lookup_thread()
+shared_ptr<sinsp_threadinfo> sinsp_threadinfo::lookup_thread() const
 {
-	return m_inspector->get_thread_ref(m_pid, true, true);
+	return m_inspector->get_thread_ref(m_pid, true, true, true);
 }
 
-//
-// Note: this is duplicated here because visual studio has trouble inlining
-//       the method.
-//
-#if defined(_WIN64) || defined(WIN64) || defined(_WIN32) || defined(WIN32)
-sinsp_threadinfo* sinsp_threadinfo::get_main_thread()
-{
-	auto main_thread = m_main_thread.lock();
-	if (!main_thread)
-	{
-		//
-		// Is this a child thread?
-		//
-		if (m_pid == m_tid)
-		{
-			//
-			// No, this is either a single thread process or the root thread of a
-			// multithread process.
-			// Note: we don't set m_main_thread because there are cases in which this is
-			//       invoked for a threadinfo that is in the stack. Caching the this pointer
-			//       would cause future mess.
-			//
-			return this;
-		}
-		else
-		{
-			//
-			// Yes, this is a child thread. Find the process root thread.
-			//
-			auto ptinfo = lookup_thread();
-			if (!ptinfo)
-			{
-				return NULL;
-			}
-			m_main_thread = ptinfo;
-			return &*ptinfo;
-		}
-	}
-
-	return &*main_thread;
-}
-#endif
-
 size_t sinsp_threadinfo::args_len() const
 {
 	return strvec_len(m_args);
@@ -1349,7 +1312,6 @@ void sinsp_thread_manager::remove_thread(int64_t tid, bool force)
 
 			erase_fd_params eparams;
 			eparams.m_remove_from_table = false;
-			eparams.m_inspector = m_inspector;
 			eparams.m_tinfo = tinfo;
 			eparams.m_ts = m_inspector->m_lastevent_ts;
 
