@@ -954,11 +954,11 @@ void sinsp_utils::ts_to_string(uint64_t ts, OUT string* res, bool date, bool ns)
 	*res = buf;
 }
 
+#define TS_STR_FMT "YYYY-MM-DDTHH:MM:SS-0000"
 void sinsp_utils::ts_to_iso_8601(uint64_t ts, OUT string* res)
 {
-	static const char *fmt = "YYYY-MM-DDTHH:MM:SS-0000";
-	static const uint32_t fmtlen = strlen(fmt);
-	char buf[fmtlen];
+	static const char *fmt = TS_STR_FMT;
+	char buf[sizeof(TS_STR_FMT)];
 	uint64_t ns = ts % ONE_SECOND_IN_NS;
 	time_t sec = ts / ONE_SECOND_IN_NS;
 
